@@ -23,13 +23,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20627 $
+ * $Revision: 21283 $
  * $Author: rjongbloed $
- * $Date: 2008-07-28 01:58:31 +0000 (Mon, 28 Jul 2008) $
+ * $Date: 2008-10-11 07:10:58 +0000 (Sat, 11 Oct 2008) $
  */
 
-#ifndef __OPALPLUGINMGR_H
-#define __OPALPLUGINMGR_H
+#ifndef OPAL_CODEC_OPALPLUGINMGR_H
+#define OPAL_CODEC_OPALPLUGINMGR_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
@@ -719,4 +719,4 @@ class H323H263PluginCapability : public H323VideoPluginCapability
 #endif // OPAL_VIDEO
 #endif // OPAL_H323
 
-#endif // __OPALPLUGINMGR_H
+#endif // OPAL_CODEC_OPALPLUGINMGR_H
