@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21283 $
- * $Author: rjongbloed $
- * $Date: 2008-10-11 07:10:58 +0000 (Sat, 11 Oct 2008) $
+ * $Revision: 21389 $
+ * $Author: hfriederich $
+ * $Date: 2008-10-20 21:14:44 +0000 (Mon, 20 Oct 2008) $
  */
 
 #ifndef OPAL_H323_H323CAPS_H
@@ -2337,6 +2337,8 @@ class H323Capabilities : public PObject
   //@}
 
   protected:
+    void SetMediaPacketizations(const PStringSet & packetizations) { mediaPacketizations = packetizations; }
+    
     H323CapabilitiesList table;
     H323CapabilitiesSet  set;
     PStringSet           mediaPacketizations;
