@@ -27,13 +27,13 @@
  *     US Joint Forces Command Joint Concept Development & Experimentation (J9)
  *     http://www.jfcom.mil/about/abt_j9.htm
  *
- * $Revision: 21004 $
- * $Author: rjongbloed $
- * $Date: 2008-09-16 07:08:56 +0000 (Tue, 16 Sep 2008) $
+ * $Revision: 21851 $
+ * $Author: csoutheren $
+ * $Date: 2008-12-22 01:35:12 +0000 (Mon, 22 Dec 2008) $
  */
 
-#ifndef __OPAL_CONNECTION_H
-#define __OPAL_CONNECTION_H
+#ifndef OPAL_OPAL_CONNECTION_H
+#define OPAL_OPAL_CONNECTION_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
@@ -48,6 +48,7 @@
 #include <ptclib/dtmf.h>
 #include <ptlib/safecoll.h>
 #include <rtp/rtp.h>
+#include <im/t140.h>
 
 class OpalEndPoint;
 class OpalCall;
@@ -861,7 +862,8 @@ class OpalConnection : public PSafeObject
        Locates a stream given an identifier string.
       */
     OpalMediaStreamPtr GetMediaStream(
-      const PString & streamID  ///<  Stream ID to search for.
+      const PString & streamID,  ///<  Stream ID to search for.
+      bool source                ///<  Indicates the direction of stream.
     ) const;
 
     /**Get a media stream.
@@ -1343,25 +1345,17 @@ class OpalConnection : public PSafeObject
 #endif
   //@}
 
-    StringOptions * GetStringOptions() const
-    { return stringOptions; }
+    /// Get the string options associated with this connection.
+    const StringOptions & GetStringOptions() const { return m_connStringOptions; }
 
-    void SetStringOptions(StringOptions * options);
+    /// Set the string options associated with this connection.
+    void SetStringOptions(
+      const StringOptions & options,
+      bool overwrite
+    );
 
-    /**Open the media channels associated with an incoming call
- 
-       This function is provided to allow an OpalConnection descendant to delay the opening of the
-       media channels associated with an incoming call. By default, this function is called as soon
-       as possible after an incoming connection request has been received (i.e. an INVITE or SETUP) and
-       and do all of the work associated with handling SDP or fastStart eyc
-
-       By overriding this function, the media channel open is deferred, which gives the other connections
-       in the call a chance to do something before the media channels are started. This allows for features
-       beyond those provided by the OnAnswer interface
-      */
-    virtual PBoolean OnOpenIncomingMediaChannels();
-
-    virtual void ApplyStringOptions();
+    virtual void ApplyStringOptions(OpalConnection::StringOptions & stringOptions);
+    virtual void OnApplyStringOptions();
 
     virtual void PreviewPeerMediaFormats(const OpalMediaFormatList & fmts);
 
@@ -1382,6 +1376,32 @@ class OpalConnection : public PSafeObject
       */
     PDECLARE_NOTIFIER(OpalMediaCommand, OpalConnection, OnMediaCommand);
 
+    /** Indicate whether a particular media type can auto-start.
+        This is typically used for things like video or fax to contol if on
+        initial connection, that media type is opened straight away. Streams
+        of that type may be opened later, during the call, by using the
+        OpalCall::OpenSourceMediaStreams() function.
+      */
+    virtual OpalMediaType::AutoStartMode GetAutoStart(
+      const OpalMediaType & mediaType  ///< media type to check
+    ) const;
+
+#if OPAL_HAS_IM
+    struct IMInfo : public PObject {
+      unsigned sessionId;
+      OpalMediaFormat mediaFormat;
+      T140String body;
+    };
+
+    virtual bool SendIM(const OpalMediaFormat & format, const T140String & body);
+    virtual bool SendIM(const OpalMediaFormat & format, const RTP_DataFrame & body);
+    virtual void OnReceiveIM(const IMInfo & im);
+
+    void AddIMListener(
+      const PNotifier & listener
+    );
+#endif
+
   protected:
     void OnConnectedInternal();
 
@@ -1452,15 +1472,41 @@ class OpalConnection : public PSafeObject
     friend ostream & operator<<(ostream & o, Phases p);
 #endif
 
-    StringOptions * stringOptions;
+    StringOptions m_connStringOptions;
     PString recordAudioFilename;
+    PNotifier recordNotifier;
 
 #if OPAL_STATISTICS
     unsigned m_VideoUpdateRequestsSent;
 #endif
+
+    struct AutoStartInfo {
+      unsigned preferredSessionId;  // preferred session ID (only used for originating)
+      OpalMediaType::AutoStartMode autoStart;// Mode for this session when the call is started
+    };
+
+    class AutoStartMap : public std::map<OpalMediaType, AutoStartInfo>
+    {
+      public:
+        AutoStartMap();
+        void Initialise(const OpalConnection::StringOptions & stringOptions);
+        OpalMediaType::AutoStartMode GetAutoStart(const OpalMediaType & mediaType) const;
+        void SetAutoStart(const OpalMediaType & mediaType, OpalMediaType::AutoStartMode autoStart);
+
+      protected:
+        bool m_initialised;
+        PMutex m_mutex;
+
+    };
+    AutoStartMap m_autoStartInfo;
+
+#if OPAL_HAS_IM
+    mutable PList<PNotifier> m_imListeners;
+#endif
+
 };
 
-#endif // __OPAL_CONNECTION_H
+#endif // OPAL_OPAL_CONNECTION_H
 
 
 // End of File ///////////////////////////////////////////////////////////////
