@@ -24,13 +24,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20867 $
- * $Author: dsandras $
- * $Date: 2008-09-05 11:11:56 +0000 (Fri, 05 Sep 2008) $
+ * $Revision: 21833 $
+ * $Author: rjongbloed $
+ * $Date: 2008-12-18 05:15:31 +0000 (Thu, 18 Dec 2008) $
  */
 
-#ifndef __OPAL_MANAGER_H
-#define __OPAL_MANAGER_H
+#ifndef OPAL_OPAL_MANAGER_H
+#define OPAL_OPAL_MANAGER_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
@@ -144,10 +144,7 @@ class OpalManager : public PObject
        application to gain access to the call at later time. This is necesary
        as any pointer being returned could become invalid (due to being
        deleted) at any time due to the multithreaded nature of the OPAL
-       system. While this function does return a pointer it is only safe to
-       use immediately after the function returns. At any other time the
-       FindCallWithLock() function should be used to gain a locked pointer
-       to the call.
+       system. 
      */
     virtual PBoolean SetUpCall(
       const PString & partyA,       ///<  The A party of call
@@ -869,21 +866,27 @@ class OpalManager : public PObject
 
 #if OPAL_VIDEO
 
+    //
+    // these functions are deprecated and used only for backwards compatibility
+    // applications should use OpalConnection::GetAutoStart() to check whether
+    // a specific media type can be auto-started
+    //
+
     /**See if should auto-start receive video channels on connection.
      */
-    PBoolean CanAutoStartReceiveVideo() const { return autoStartReceiveVideo; }
+    bool CanAutoStartReceiveVideo() const { return (OpalMediaType::Video().GetAutoStart()&OpalMediaType::Receive) != 0; }
 
     /**Set if should auto-start receive video channels on connection.
      */
-    void SetAutoStartReceiveVideo(PBoolean can) { autoStartReceiveVideo = can; }
+    void SetAutoStartReceiveVideo(bool can) { OpalMediaType::Video().GetDefinition()->SetAutoStart(CanAutoStartTransmitVideo() ? can ? OpalMediaType::ReceiveTransmit : OpalMediaType::Transmit : can ? OpalMediaType::Receive : OpalMediaType::OfferInactive); }
 
     /**See if should auto-start transmit video channels on connection.
      */
-    PBoolean CanAutoStartTransmitVideo() const { return autoStartTransmitVideo; }
+    bool CanAutoStartTransmitVideo() const { return (OpalMediaType::Video().GetAutoStart()&OpalMediaType::Transmit) != 0; }
 
     /**Set if should auto-start transmit video channels on connection.
      */
-    void SetAutoStartTransmitVideo(PBoolean can) { autoStartTransmitVideo = can; }
+    void SetAutoStartTransmitVideo(bool can) { OpalMediaType::Video().GetDefinition()->SetAutoStart(CanAutoStartReceiveVideo() ? can ? OpalMediaType::ReceiveTransmit : OpalMediaType::Receive : can ? OpalMediaType::Transmit : OpalMediaType::OfferInactive); }
 
 #endif
 
@@ -1207,14 +1210,6 @@ class OpalManager : public PObject
       OpalConnection & connection   ///< New connection just created
     );
 
-    virtual void SetDefaultSecurityMode(const PString & v)
-    { defaultSecurityMode = v; }
-
-    virtual PString GetDefaultSecurityMode() const 
-    { return defaultSecurityMode; }
-
-    virtual PBoolean UseRTPAggregation() const;
-
     OpalRecordManager & GetRecordManager()
     { return recordManager; }
 
@@ -1222,6 +1217,20 @@ class OpalManager : public PObject
     virtual bool IsRecording(const PString & callToken);
     virtual void StopRecording(const PString & callToken);
 
+#ifdef OPAL_ZRTP
+    virtual bool GetZRTPEnabled() const;
+#endif
+
+    virtual void AddIMMediaFormats(
+      OpalMediaFormatList & mediaFormats,       ///<  Media formats to use
+      const OpalConnection * connection = NULL  ///<  Optional connection that is using formats
+    ) const;
+
+    virtual void OnApplyStringOptions(
+      OpalConnection & conn,
+      OpalConnection::StringOptions & stringOptions
+    );
+
   protected:
     // Configuration variables
     OpalProductInfo productInfo;
@@ -1229,11 +1238,6 @@ class OpalManager : public PObject
     PString       defaultUserName;
     PString       defaultDisplayName;
 
-#if OPAL_VIDEO
-    PBoolean          autoStartReceiveVideo;
-    PBoolean          autoStartTransmitVideo;
-#endif
-
     BYTE          rtpIpTypeofService;
     PINDEX        rtpPayloadSizeMax;
     unsigned      minAudioJitterDelay;
@@ -1317,10 +1321,8 @@ class OpalManager : public PObject
     PSyncPoint   garbageCollectExit;
     PDECLARE_NOTIFIER(PThread, OpalManager, GarbageMain);
 
-    PString defaultSecurityMode;
-
-#if OPAL_RTP_AGGREGATE
-    PBoolean useRTPAggregation; 
+#ifdef OPAL_ZRTP
+    bool zrtpEnabled;
 #endif
 
     OpalRecordManager recordManager;
@@ -1336,7 +1338,7 @@ unsigned OpalGetMinorVersion();
 unsigned OpalGetBuildNumber();
 
 
-#endif // __OPAL_MANAGER_H
+#endif // OPAL_OPAL_MANAGER_H
 
 
 // End of File ///////////////////////////////////////////////////////////////
