@@ -24,98 +24,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Log: transcoders.h,v $
- * Revision 2.21.2.1  2006/02/06 04:38:37  csoutheren
- * Backported RTP payload mapping fixes from CVS head
- *
- * Revision 2.22  2006/02/02 07:02:57  csoutheren
- * Added RTP payload map to transcoders and connections to allow remote SIP endpoints
- * to change the payload type used for outgoing RTP.
- *
- * Revision 2.21  2005/12/30 14:33:12  dsandras
- * Added support for Packet Loss Concealment frames for framed codecs supporting it similarly to what was done for OpenH323.
- *
- * Revision 2.20  2005/11/30 13:35:26  csoutheren
- * Changed tags for Doxygen
- *
- * Revision 2.19  2005/09/06 12:44:49  rjongbloed
- * Many fixes to finalise the video processing: merging remote media
- *
- * Revision 2.18  2005/09/04 06:23:38  rjongbloed
- * Added OpalMediaCommand mechanism (via PNotifier) for media streams
- *   and media transcoders to send commands back to remote.
- *
- * Revision 2.17  2005/09/02 14:31:40  csoutheren
- * Use inline function to work around compiler foo in gcc
- *
- * Revision 2.16  2005/08/31 13:19:25  rjongbloed
- * Added mechanism for controlling media (especially codecs) including
- *   changing the OpalMediaFormat option list (eg bit rate) and a completely
- *   new OpalMediaCommand abstraction for things like video fast update.
- *
- * Revision 2.15  2005/08/28 07:59:17  rjongbloed
- * Converted OpalTranscoder to use factory, requiring sme changes in making sure
- *   OpalMediaFormat instances are initialised before use.
- *
- * Revision 2.14  2005/03/19 04:08:10  csoutheren
- * Fixed warnings with gcc snapshot 4.1-20050313
- * Updated to configure 2.59
- *
- * Revision 2.13  2005/02/17 03:25:05  csoutheren
- * Added support for audio codecs that consume and produce variable size
- * frames, such as G.723.1
- *
- * Revision 2.12  2004/07/11 12:34:48  rjongbloed
- * Added function to get a list of all possible media formats that may be used given
- *   a list of media and taking into account all of the registered transcoders.
- *
- * Revision 2.11  2004/03/22 11:32:41  rjongbloed
- * Added new codec type for 16 bit Linear PCM as must distinguish between the internal
- *   format used by such things as the sound card and the RTP payload format which
- *   is always big endian.
- *
- * Revision 2.10  2004/03/11 06:54:27  csoutheren
- * Added ability to disable SIP or H.323 stacks
- *
- * Revision 2.9  2004/02/17 08:47:38  csoutheren
- * Changed codec loading macros to work with Linux
- *
- * Revision 2.8  2004/01/18 15:35:20  rjongbloed
- * More work on video support
- *
- * Revision 2.7  2003/06/02 02:59:43  rjongbloed
- * Changed transcoder search so uses destination list as preference order.
- *
- * Revision 2.6  2003/03/24 04:32:11  robertj
- * Fixed macro for transcoder with parameter (not used yet!)
- * Fixed so OPAL_NO_PARAM can be defined in other modules.
- *
- * Revision 2.5  2003/03/17 10:26:59  robertj
- * Added video support.
- *
- * Revision 2.4  2002/09/16 02:52:35  robertj
- * Added #define so can select if #pragma interface/implementation is used on
- *   platform basis (eg MacOS) rather than compiler, thanks Robert Monaghan.
- *
- * Revision 2.3  2002/02/13 02:30:21  robertj
- * Added ability for media patch (and transcoders) to handle multiple RTP frames.
- *
- * Revision 2.2  2002/01/22 05:07:02  robertj
- * Added ability to get input and output media format names from transcoder.
- *
- * Revision 2.1  2001/08/01 05:52:08  robertj
- * Made OpalMediaFormatList class global to help with documentation.
- * Added functions to aid in determining if a transcoder can be used to get
- *   to another media format.
- * Fixed problem with streamed transcoder used in G.711.
- *
- * Revision 2.0  2001/07/27 15:48:24  robertj
- * Conversion of OpenH323 to Open Phone Abstraction Library (OPAL)
- *
+ * $Revision: 21283 $
+ * $Author: rjongbloed $
+ * $Date: 2008-10-11 07:10:58 +0000 (Sat, 11 Oct 2008) $
  */
 
-#ifndef __OPAL_TRANSCODERS_H
-#define __OPAL_TRANSCODERS_H
+#ifndef OPAL_OPAL_TRANSCODERS_H
+#define OPAL_OPAL_TRANSCODERS_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
@@ -194,15 +109,23 @@ class OpalMediaFormatPair : public PObject
 };
 
 
-typedef PFactory<OpalTranscoder, OpalMediaFormatPair>                      OpalTranscoderFactory;
-typedef PFactory<OpalTranscoder, OpalMediaFormatPair>::KeyList_T           OpalTranscoderList;
-typedef PFactory<OpalTranscoder, OpalMediaFormatPair>::KeyList_T::iterator OpalTranscoderIterator;
+typedef std::pair<PString, PString>                                      OpalTranscoderKey;
+typedef PFactory<OpalTranscoder, OpalTranscoderKey>                      OpalTranscoderFactory;
+typedef PFactory<OpalTranscoder, OpalTranscoderKey>::KeyList_T           OpalTranscoderList;
+typedef PFactory<OpalTranscoder, OpalTranscoderKey>::KeyList_T::iterator OpalTranscoderIterator;
 
-inline OpalMediaFormatPair OpalCreateMediaFormatPair(const OpalMediaFormat & from, const OpalMediaFormat & to)
-{ return OpalMediaFormatPair(from, to); }
+__inline OpalTranscoderKey MakeOpalTranscoderKey(const OpalMediaFormat & from, const OpalMediaFormat & to)
+{
+  return OpalTranscoderKey(from.GetName(), to.GetName());
+}
+
+__inline OpalTranscoderKey MakeOpalTranscoderKey(const char * from, const char * to)
+{
+  return OpalTranscoderKey(from, to);
+}
 
 #define OPAL_REGISTER_TRANSCODER(cls, input, output) \
-  OpalTranscoderFactory::Worker<cls> OpalTranscoder_##cls(OpalCreateMediaFormatPair(input, output))
+  OpalTranscoderFactory::Worker<cls> OpalTranscoder_##cls(MakeOpalTranscoderKey(input, output))
 
 
 /**This class embodies the implementation of a specific transcoder instance
@@ -227,25 +150,31 @@ class OpalTranscoder : public OpalMediaFormatPair
 
   /**@name Operations */
   //@{
-    /**Update the output media format. This can be used to adjust the
-       parameters of a codec at run time. Note you cannot change the basic
+    /**Update the input and output media formats. This can be used to adjust
+       the parameters of a codec at run time. Note you cannot change the basic
        media format, eg change GSM0610 to G.711, only options for that
-       format, eg 6k3 mode to 5k3 mode in G.723.1.
+       format, eg 6k3 mode to 5k3 mode in G.723.1. If the formats are
+       different then a OpalMediaFormat::Merge() is performed.
 
-       The default behaviour updates the outputMediaFormat member variable
-       and sets the outputMediaFormatUpdated flag.
+       If a format is empty (invalid) it is ignored and does not update the
+       internal variable. In this way only the input or output side can be
+       updated.
+
+       The default behaviour updates the inputMediaFormat and outputMediaFormat
+       member variables.
       */
-    virtual BOOL UpdateOutputMediaFormat(
-      const OpalMediaFormat & mediaFormat  ///<  New media format
+    virtual bool UpdateMediaFormats(
+      const OpalMediaFormat & inputMediaFormat,  ///<  Input media format
+      const OpalMediaFormat & outputMediaFormat  ///<  Output media format
     );
 
     /**Execute the command specified to the transcoder. The commands are
        highly context sensitive, for example VideoFastUpdate would only apply
        to a video transcoder.
 
-       The default behaviour simply returns FALSE.
+       The default behaviour simply returns PFalse.
       */
-    virtual BOOL ExecuteCommand(
+    virtual PBoolean ExecuteCommand(
       const OpalMediaCommand & command    ///<  Command to execute.
     );
 
@@ -256,7 +185,7 @@ class OpalTranscoder : public OpalMediaFormatPair
        must be able to handle any sized packets.
       */
     virtual PINDEX GetOptimalDataFrameSize(
-      BOOL input      ///<  Flag for input or output data size
+      PBoolean input      ///<  Flag for input or output data size
     ) const = 0;
 
     /**Convert the data from one format to another.
@@ -267,9 +196,9 @@ class OpalTranscoder : public OpalMediaFormatPair
        The default behaviour makes sure the output list has only one element
        in it and calls the Convert() function.
 
-       Returns FALSE if the conversion fails.
+       Returns PFalse if the conversion fails.
       */
-    virtual BOOL ConvertFrames(
+    virtual PBoolean ConvertFrames(
       const RTP_DataFrame & input,  ///<  Input data
       RTP_DataFrameList & output    ///<  Output data
     );
@@ -278,9 +207,9 @@ class OpalTranscoder : public OpalMediaFormatPair
        This function takes the input data as a RTP_DataFrame and converts it
        to its output format, placing it into the RTP_DataFrame provided.
 
-       Returns FALSE if the conversion fails.
+       Returns PFalse if the conversion fails.
       */
-    virtual BOOL Convert(
+    virtual PBoolean Convert(
       const RTP_DataFrame & input,  ///<  Input data
       RTP_DataFrame & output        ///<  Output data
     ) = 0;
@@ -291,7 +220,9 @@ class OpalTranscoder : public OpalMediaFormatPair
       */
     static OpalTranscoder * Create(
       const OpalMediaFormat & srcFormat,  ///<  Name of source format
-      const OpalMediaFormat & dstFormat   ///<  Name of destination format
+      const OpalMediaFormat & dstFormat,  ///<  Name of destination format
+      const BYTE * instance = NULL,       ///<  Unique instance identifier for transcoder
+      unsigned instanceLen = 0            ///<  Length of instance identifier
     );
 
     /**Find media format(s) for transcoders.
@@ -302,15 +233,15 @@ class OpalTranscoder : public OpalMediaFormatPair
        There could be many possible matches between the two lists, so
        preference is given to the order of the destination formats.
 
-       Returns FALSE if there is no registered media transcoder that can be used
+       Returns PFalse if there is no registered media transcoder that can be used
        between the two named formats.
       */
-    static BOOL SelectFormats(
-      unsigned sessionID,               ///<  Session ID for media formats
+    static bool SelectFormats(
       const OpalMediaFormatList & srcFormats, ///<  Names of possible source formats
       const OpalMediaFormatList & dstFormats, ///<  Names of possible destination formats
-      OpalMediaFormat & srcFormat,      ///<  Selected source format to be used
-      OpalMediaFormat & dstFormat       ///<  Selected destination format to be used
+      const OpalMediaFormatList & allFormats, ///<  Master list of formats for merging options
+      OpalMediaFormat & srcFormat,            ///<  Selected source format to be used
+      OpalMediaFormat & dstFormat             ///<  Selected destination format to be used
     );
 
     /**Find media intermediate format for transcoders.
@@ -318,10 +249,14 @@ class OpalTranscoder : public OpalMediaFormatPair
        allow two transcoders to be used to get data from the source format to
        the destination format.
 
-       Returns FALSE if there is no registered media transcoder that can be used
+       If there is a transcoder that can go directly from the source format to
+       the destination format then the function returns true but the
+       intermediateFormat parmaeter will be an invlid format.
+
+       Returns false if there is no registered media transcoder that can be used
        between the two named formats.
       */
-    static BOOL FindIntermediateFormat(
+    static bool FindIntermediateFormat(
       const OpalMediaFormat & srcFormat,    ///<  Selected destination format to be used
       const OpalMediaFormat & dstFormat,    ///<  Selected destination format to be used
       OpalMediaFormat & intermediateFormat  ///<  Intermediate format that can be used
@@ -371,22 +306,35 @@ class OpalTranscoder : public OpalMediaFormatPair
        would only apply to a video transcoder.
       */
     const PNotifier & GetCommandNotifier() const { return commandNotifier; }
+
+    /** Set the unique instance identifier for transcoder
+      */
+    virtual void SetInstanceID(
+      const BYTE * instance,              ///<  Unique instance identifier for transcoder
+      unsigned instanceLen                ///<  Length of instance identifier
+    );
+
+    RTP_DataFrame::PayloadTypes GetPayloadType(
+      PBoolean input      ///<  Flag for input or output data size
+    ) const;
+
+    virtual bool AcceptComfortNoise() const  { return false; }
+    virtual bool AcceptEmptyPayload() const  { return acceptEmptyPayload; }
+    virtual bool AcceptOtherPayloads() const { return acceptOtherPayloads; }
+
+#if OPAL_STATISTICS
+    virtual void GetStatistics(OpalMediaStatistics & statistics) const;
+#endif
   //@}
 
   protected:
     PINDEX    maxOutputSize;
-    bool      outputMediaFormatUpdated;
     PNotifier commandNotifier;
     PMutex    updateMutex;
 
-    RTP_DataFrame::PayloadMapType payloadTypeMap;
-
-  public:
-    void SetRTPPayloadMap(const RTP_DataFrame::PayloadMapType & v)
-    { payloadTypeMap = v; }
-
-    void AddRTPPayloadMapping(RTP_DataFrame::PayloadTypes from, RTP_DataFrame::PayloadTypes to)
-    { payloadTypeMap.insert(RTP_DataFrame::PayloadMapType::value_type(from, to)); }
+    PBoolean outputIsRTP, inputIsRTP;
+    bool acceptEmptyPayload;
+    bool acceptOtherPayloads;
 };
 
 
@@ -415,6 +363,24 @@ class OpalFramedTranscoder : public OpalTranscoder
 
   /**@name Operations */
   //@{
+    /**Update the input and output media formats. This can be used to adjust
+       the parameters of a codec at run time. Note you cannot change the basic
+       media format, eg change GSM0610 to G.711, only options for that
+       format, eg 6k3 mode to 5k3 mode in G.723.1. If the formats are
+       different then a OpalMediaFormat::Merge() is performed.
+
+       If a format is empty (invalid) it is ignored and does not update the
+       internal variable. In this way only the input or output side can be
+       updated.
+
+       The default behaviour updates the inputMediaFormat and outputMediaFormat
+       member variables.
+      */
+    virtual bool UpdateMediaFormats(
+      const OpalMediaFormat & inputMediaFormat,  ///<  Input media format
+      const OpalMediaFormat & outputMediaFormat  ///<  Output media format
+    );
+
     /**Get the optimal size for data frames to be converted.
        This function returns the size of frames that will be most efficient
        in conversion. A RTP_DataFrame will attempt to provide or use data in
@@ -422,7 +388,7 @@ class OpalFramedTranscoder : public OpalTranscoder
        must be able to handle any sized packets.
       */
     virtual PINDEX GetOptimalDataFrameSize(
-      BOOL input      ///<  Flag for input or output data size
+      PBoolean input      ///<  Flag for input or output data size
     ) const;
 
     /**Convert the data from one format to another.
@@ -431,7 +397,7 @@ class OpalFramedTranscoder : public OpalTranscoder
 
        Returns FALSE if the conversion fails.
       */
-    virtual BOOL Convert(
+    virtual PBoolean Convert(
       const RTP_DataFrame & input,  ///<  Input data
       RTP_DataFrame & output        ///<  Output data
     );
@@ -439,17 +405,17 @@ class OpalFramedTranscoder : public OpalTranscoder
     /**Convert a frame of data from one format to another.
        This function implicitly knows the input and output frame sizes.
       */
-    virtual BOOL ConvertFrame(
+    virtual PBoolean ConvertFrame(
       const BYTE * input,   ///<  Input data
       BYTE * output         ///<  Output data
     );
-    virtual BOOL ConvertFrame(
+    virtual PBoolean ConvertFrame(
       const BYTE * input,   ///<  Input data
       PINDEX & consumed,    ///<  number of input bytes consumed
       BYTE * output,        ///<  Output data
-      PINDEX & created      ///<  number of output bytes created  
+      PINDEX & created      ///<  number of output bytes created
     );
-    virtual BOOL ConvertSilentFrame(
+    virtual PBoolean ConvertSilentFrame(
       BYTE * output         ///<  Output data
     );
   //@}
@@ -457,8 +423,7 @@ class OpalFramedTranscoder : public OpalTranscoder
   protected:
     PINDEX     inputBytesPerFrame;
     PINDEX     outputBytesPerFrame;
-    //PBYTEArray partialFrame;
-    //PINDEX     partialBytes;
+    PINDEX     maxOutputDataSize;
 };
 
 
@@ -481,8 +446,7 @@ class OpalStreamedTranscoder : public OpalTranscoder
       const OpalMediaFormat & inputMediaFormat,  ///<  Input media format
       const OpalMediaFormat & outputMediaFormat, ///<  Output media format
       unsigned inputBits,           ///<  Bits per sample in input data
-      unsigned outputBits,          ///<  Bits per sample in output data
-      PINDEX   optimalSamples       ///<  Optimal number of samples for read
+      unsigned outputBits           ///<  Bits per sample in output data
     );
   //@}
 
@@ -495,16 +459,16 @@ class OpalStreamedTranscoder : public OpalTranscoder
        must be able to handle any sized packets.
       */
     virtual PINDEX GetOptimalDataFrameSize(
-      BOOL input      ///<  Flag for input or output data size
+      PBoolean input      ///<  Flag for input or output data size
     ) const;
 
     /**Convert the data from one format to another.
        This function takes the input data as a RTP_DataFrame and converts it
        to its output format, placing it into the RTP_DataFrame provided.
 
-       Returns FALSE if the conversion fails.
+       Returns PFalse if the conversion fails.
       */
-    virtual BOOL Convert(
+    virtual PBoolean Convert(
       const RTP_DataFrame & input,  ///<  Input data
       RTP_DataFrame & output        ///<  Output data
     );
@@ -521,7 +485,6 @@ class OpalStreamedTranscoder : public OpalTranscoder
   protected:
     unsigned inputBitsPerSample;
     unsigned outputBitsPerSample;
-    PINDEX   optimalSamples;
 };
 
 
@@ -550,8 +513,39 @@ class Opal_PCM_Linear16Mono : public OpalStreamedTranscoder {
   OPAL_REGISTER_TRANSCODER(Opal_PCM_Linear16Mono, OpalPCM16,         OpalL16_MONO_8KHZ)
 
 
+class OpalEmptyFramedAudioTranscoder : public OpalFramedTranscoder
+{
+  PCLASSINFO(OpalEmptyFramedAudioTranscoder, OpalFramedTranscoder);
+  public:
+    OpalEmptyFramedAudioTranscoder(const char * inFormat, const char * outFormat)
+      : OpalFramedTranscoder(inFormat, outFormat, 100, 100)
+    {  }
 
-#endif // __OPAL_TRANSCODERS_H
+    PBoolean ConvertFrame(const BYTE *, PINDEX &, BYTE *, PINDEX &)
+    { return PFalse; }
+};
+
+#define OPAL_DECLARE_EMPTY_TRANSCODER(fmt) \
+class Opal_Empty_##fmt##_Encoder : public OpalEmptyFramedAudioTranscoder \
+{ \
+  public: \
+    Opal_Empty_##fmt##_Encoder() \
+      : OpalEmptyFramedAudioTranscoder(OpalPCM16, fmt) \
+    { } \
+}; \
+class Opal_Empty_##fmt##_Decoder : public OpalEmptyFramedAudioTranscoder \
+{ \
+  public: \
+    Opal_Empty_##fmt##_Decoder() \
+      : OpalEmptyFramedAudioTranscoder(fmt, OpalPCM16) \
+    { } \
+}; \
+
+#define OPAL_DEFINE_EMPTY_TRANSCODER(fmt) \
+OPAL_REGISTER_TRANSCODER(Opal_Empty_##fmt##_Encoder, OpalPCM16, fmt); \
+OPAL_REGISTER_TRANSCODER(Opal_Empty_##fmt##_Decoder, fmt,       OpalPCM16); \
+
+#endif // OPAL_OPAL_TRANSCODERS_H
 
 
 // End of File ///////////////////////////////////////////////////////////////
