@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21307 $
+ * $Revision: 21811 $
  * $Author: rjongbloed $
- * $Date: 2008-10-13 07:06:50 +0000 (Mon, 13 Oct 2008) $
+ * $Date: 2008-12-16 10:06:55 +0000 (Tue, 16 Dec 2008) $
  */
 
 #ifndef OPAL_SIP_SIPPDU_H
@@ -96,6 +96,11 @@ class SIPURL : public PURL
       WORD listenerPort = 0
     );
 
+    SIPURL(
+      const OpalTransportAddress & _address, 
+      WORD listenerPort = 0
+    );
+
     /**Compare the two SIPURLs and return their relative rank.
        Note that does an intelligent comparison according to the rules
        in RFC3261 Section 19.1.4.
@@ -124,7 +129,11 @@ class SIPURL : public PURL
       */
     PString GetFieldParameters() const { return fieldParameters; }
 
-    /**Get the host and port as a transpoprt address.
+    /**Returns the field parameter (outside of <>)
+      */
+    void SetFieldParameters(const PString & str ) { fieldParameters = str; }
+
+    /**Get the host and port as a transport address.
       */
     OpalTransportAddress GetHostAddress() const;
 
@@ -134,7 +143,8 @@ class SIPURL : public PURL
       ToURI,        ///< To header field
       FromURI,      ///< From header field
       ContactURI,   ///< Registration or Redirection Contact header field
-      RouteURI      ///< Dialog Contact header field, or Record-Route header field
+      RouteURI,     ///< Dialog Contact header field, or Record-Route header field
+      RegisterURI   ///< URI on REGISTER request line.
     };
 
     /** Removes tag parm & query vars and recalculates urlString
@@ -156,6 +166,8 @@ class SIPURL : public PURL
     );
 
   protected:
+    void ParseAsAddress(const PString & name, const OpalTransportAddress & _address, WORD listenerPort = 0);
+
     /** Parses name-addr, like:
         "displayname"<scheme:user:password@host:port;transport=type>;tag=value
         into:
@@ -306,9 +318,10 @@ class SIPMIMEInfo : public PMIMEInfo
     void SetProxyAuthenticate(const PString & v);
 
     PStringList GetRoute() const;
+    void SetRoute(const PString & v);
     void SetRoute(const PStringList & v);
 
-    PStringList GetRecordRoute() const;
+    PStringList GetRecordRoute(bool reversed) const;
     void SetRecordRoute(const PStringList & v);
 
     unsigned GetCSeqIndex() const { return GetCSeq().AsUnsigned(); }
@@ -343,6 +356,9 @@ class SIPMIMEInfo : public PMIMEInfo
     PString GetSIPETag() const;
     void SetSIPETag(const PString & v);
 
+    PString GetRequire() const;
+    void SetRequire(const PString & v, bool overwrite);
+
     /** return the value of a header field parameter, empty if none
      */
     PString GetFieldParameter(
@@ -382,7 +398,7 @@ class SIPMIMEInfo : public PMIMEInfo
   protected:
     	/** return list of route values from internal comma-delimited list
 	 */
-    PStringList GetRouteList(const char * name) const;
+    PStringList GetRouteList(const char * name, bool reversed) const;
 
 	/** store string list as one comma-delimited string of route values
 	    value formed as "<v[0]>,<v[1]>,<v[2]>" etc
@@ -656,7 +672,8 @@ class SIP_PDU : public PSafeObject
       If first route is strict, exchange with URI.
       Returns PTrue if routeSet.
       */
-    PBoolean SetRoute(const PStringList & routeSet);
+    bool SetRoute(const PStringList & routeSet);
+    bool SetRoute(const SIPURL & proxy);
 
     /**Set mime allow field to all supported methods.
       */
@@ -685,12 +702,14 @@ class SIP_PDU : public PSafeObject
     bool SendResponse(
       OpalTransport & transport,
       StatusCodes code,
+      SIPEndPoint * endpoint = NULL,
       const char * contact = NULL,
       const char * extra = NULL
     );
     bool SendResponse(
       OpalTransport & transport,
-      SIP_PDU & response
+      SIP_PDU & response,
+      SIPEndPoint * endpoint = NULL
     );
 
     /** Construct the PDU string to output.
@@ -737,6 +756,58 @@ ostream & operator<<(ostream & strm, SIP_PDU::Methods method);
 #endif
 
 
+/////////////////////////////////////////////////////////////////////////
+// SIPDialogContext
+
+/** Session Initiation Protocol dialog context information.
+  */
+class SIPDialogContext
+{
+  public:
+    SIPDialogContext();
+
+    const PString & GetCallID() const { return m_callId; }
+    void SetCallID(const PString & id) { m_callId = id; }
+    void GenerateCallID();
+
+    const SIPURL & GetRequestURI() const { return m_requestURI; }
+    void SetRequestURI(const SIPURL & url) { m_requestURI = url; }
+    bool SetRequestURI(const PString & uri) { return m_requestURI.Parse(uri); }
+
+    const SIPURL & GetLocalURI() const { return m_localURI; }
+    const PString & GetLocalTag() const { return m_localTag; }
+    void SetLocalURI(const SIPURL & url);
+    bool SetLocalURI(const PString & uri);
+
+    const SIPURL & GetRemoteURI() const { return m_remoteURI; }
+    const PString & GetRemoteTag() const { return m_remoteTag; }
+    void SetRemoteURI(const SIPURL & url);
+    bool SetRemoteURI(const PString & uri);
+
+    const PStringList & GetRouteSet() const { return m_routeSet; }
+    void SetRouteSet(const PStringList & routes) { m_routeSet = routes; }
+    void UpdateRouteSet(const SIPURL & proxy);
+
+    void Update(const SIP_PDU & response);
+
+    unsigned GetNextCSeq() { return ++m_lastSentCSeq; }
+    bool IsDuplicateCSeq(unsigned sequenceNumber);
+
+    bool IsEstablished() const { return !m_remoteTag.IsEmpty(); }
+
+  protected:
+    PString     m_callId;
+    SIPURL      m_requestURI;
+    SIPURL      m_localURI;
+    PString     m_localTag;
+    SIPURL      m_remoteURI;
+    PString     m_remoteTag;
+    PStringList m_routeSet;
+    unsigned    m_lastSentCSeq;
+    unsigned    m_lastReceivedCSeq;
+};
+
+
 /////////////////////////////////////////////////////////////////////////
 // SIPTransaction
 
@@ -788,11 +859,17 @@ class SIPTransaction : public SIP_PDU
     SIPConnection * GetConnection() const { return connection; }
     PString         GetInterface() const { return m_localInterface; }
 
+    static void GenerateCallID(PString & callId);
+    
   protected:
     void Construct(
       const PTimeInterval & minRetryTime = PMaxTimeInterval,
       const PTimeInterval & maxRetryTime = PMaxTimeInterval
     );
+    void Construct(
+      Methods method, 
+      SIPDialogContext & dialog
+    );
     bool SendPDU(SIP_PDU & pdu);
     bool ResendCANCEL();
 
@@ -846,12 +923,12 @@ class SIPInvite : public SIPTransaction
     SIPInvite(
       SIPConnection & connection,
       OpalTransport & transport,
-      OpalRTPSessionManager * sm
+      const OpalRTPSessionManager & sm
     );
 
     virtual PBoolean OnReceivedResponse(SIP_PDU & response);
 
-    OpalRTPSessionManager & GetSessionManager() { return rtpSessions; }
+    const OpalRTPSessionManager & GetSessionManager() const { return rtpSessions; }
 
   protected:
     OpalRTPSessionManager rtpSessions;
@@ -882,8 +959,8 @@ class SIPRegister : public SIPTransaction
     SIPRegister(
       SIPEndPoint   & endpoint,
       OpalTransport & transport,
-      const PStringList & routeSet,
-      const PString & id,
+      const SIPURL & proxy,
+      const PString & callId,
       unsigned cseq,
       const Params & params
     );
@@ -901,17 +978,29 @@ class SIPSubscribe : public SIPTransaction
     enum PredefinedPackages {
       MessageSummary,
       Presence,
+      Dialog,
       NumPredefinedPackages
     };
-    static PredefinedPackages GetEventPackage(const PString & name);
-    static PString GetEventPackageName(PredefinedPackages pkg);
+
+    class EventPackage : public PCaselessString
+    {
+        PCLASSINFO(EventPackage, PCaselessString);
+      public:
+        EventPackage(PredefinedPackages);
+        EventPackage(const PString & str) : PCaselessString(str) { }
+        EventPackage(const char * cstr) : PCaselessString(cstr) { }
+        bool operator==(PredefinedPackages) const;
+        bool operator==(const PString & str) const { return Compare(str) == EqualTo; }
+        bool operator==(const char * cstr) const { return InternalCompare(0, P_MAX_INDEX, cstr) == EqualTo; }
+        virtual Comparison InternalCompare(PINDEX offset, PINDEX length, const char * cstr) const;
+    };
 
     struct Params {
       Params(PredefinedPackages pkg = NumPredefinedPackages);
 
-      PString       m_eventPackage;
-      PString       m_mimeType;
-      PString       m_targetAddress;
+      EventPackage  m_eventPackage;
+      PString       m_agentAddress;
+      PString       m_addressOfRecord;
       PString       m_contactAddress;
       PString       m_authID;
       PString       m_password;
@@ -922,21 +1011,51 @@ class SIPSubscribe : public SIPTransaction
       PTimeInterval m_maxRetryTime;
     };
 
-    /** Valid types for a MWI
-    */
     SIPSubscribe(
         SIPEndPoint & ep,
         OpalTransport & trans,
-        const PStringList & routeSet,
-        const PString & to,
-        const PString & from,
-        const PString & id,
-        unsigned cseq,
+        SIPDialogContext & dialog,
         const Params & params
     );
 };
 
 
+typedef SIPSubscribe::EventPackage SIPEventPackage;
+
+
+/////////////////////////////////////////////////////////////////////////
+
+class SIPHandler;
+
+class SIPEventPackageHandler
+{
+public:
+  virtual ~SIPEventPackageHandler() { }
+  virtual PString GetContentType() const = 0;
+  virtual bool OnReceivedNOTIFY(SIPHandler & handler, SIP_PDU & request) = 0;
+};
+
+
+typedef PFactory<SIPEventPackageHandler, SIPEventPackage> SIPEventPackageFactory;
+
+
+/////////////////////////////////////////////////////////////////////////
+
+class SIPNotify : public SIPTransaction
+{
+    PCLASSINFO(SIPNotify, SIPTransaction);
+  public:
+    SIPNotify(
+        SIPEndPoint & ep,
+        OpalTransport & trans,
+        SIPDialogContext & dialog,
+        const SIPEventPackage & eventPackage,
+        const PString & state,
+        const PString & body
+    );
+};
+
+
 /////////////////////////////////////////////////////////////////////////
 
 class SIPPublish : public SIPTransaction
@@ -946,12 +1065,10 @@ class SIPPublish : public SIPTransaction
     SIPPublish(
       SIPEndPoint & ep,
       OpalTransport & trans,
-      const PStringList & routeSet,
-      const SIPURL & targetAddress,
       const PString & id,
       const PString & sipIfMatch,
-      const PString & body,
-      unsigned expires
+      SIPSubscribe::Params & params,
+      const PString & body
     );
 };
 
@@ -1010,12 +1127,12 @@ class SIPMessage : public SIPTransaction
     
   public:
     SIPMessage(
-	       SIPEndPoint & ep,
-	       OpalTransport & trans,
-	       const SIPURL & to,
-               const PStringList & routeSet,
-               const PString & id,
-	       const PString & body
+      SIPEndPoint & ep,
+      OpalTransport & trans,
+      const SIPURL & proxy,
+      const SIPURL & to,
+      const PString & id,
+      const PString & body
     );
 };
 
