@@ -23,13 +23,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20627 $
+ * $Revision: 21857 $
  * $Author: rjongbloed $
- * $Date: 2008-07-28 03:58:31 +0200 (lun 28 jui 2008) $
+ * $Date: 2008-12-22 04:03:10 +0000 (Mon, 22 Dec 2008) $
  */
 
-#ifndef __OPAL_T38PROTO_H
-#define __OPAL_T38PROTO_H
+#ifndef OPAL_T38_T38PROTO_H
+#define OPAL_T38_T38PROTO_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
@@ -50,6 +50,7 @@
 class OpalTransport;
 class T38_IFPPacket;
 class PASN_OctetString;
+class OpalFaxConnection;
 
 
 namespace PWLibStupidLinkerHacks {
@@ -108,13 +109,13 @@ class OpalFaxMediaStream : public OpalMediaStream
     /**Construct a new media stream for T.38 sessions.
       */
     OpalFaxMediaStream(
-      OpalConnection & conn,
+      OpalFaxConnection & conn,
       const OpalMediaFormat & mediaFormat, ///<  Media format for stream
       unsigned sessionID, 
-      PBoolean isSource ,                      ///<  Is a source stream
+      bool isSource,                       ///<  Is a source stream
       const PString & token,               ///<  token used to match incoming/outgoing streams
       const PString & filename,
-      PBoolean receive,
+      bool  receive,
       const PString & stationId
     );
   //@}
@@ -133,13 +134,6 @@ class OpalFaxMediaStream : public OpalMediaStream
       */
     virtual PBoolean Close();
 
-    /**Start the media stream.
-
-       The default behaviour calls Resume() on the associated OpalMediaPatch
-       thread if it was suspended.
-      */
-    virtual PBoolean Start();
-
     /**Read an RTP frame of data from the source media stream.
        The new behaviour simply calls RTP_Session::ReadData().
       */
@@ -164,14 +158,15 @@ class OpalFaxMediaStream : public OpalMediaStream
   //@}
 
   protected:
-    PMutex infoMutex;
-    PString sessionToken;
-    OpalFaxCallInfo * faxCallInfo;
-    PFilePath filename;
-    PBoolean receive;
-    BYTE writeBuffer[320];
-    PINDEX writeBufferLen;
-    PString stationId;
+    OpalFaxConnection & m_connection;
+    PMutex              infoMutex;
+    PString             sessionToken;
+    OpalFaxCallInfo   * faxCallInfo;
+    PFilePath           m_filename;
+    bool                m_receive;
+    BYTE                writeBuffer[320];
+    PINDEX              writeBufferLen;
+    PString             m_stationId;
 };
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -183,20 +178,20 @@ class OpalT38MediaStream : public OpalFaxMediaStream
   PCLASSINFO(OpalT38MediaStream, OpalFaxMediaStream);
   public:
     OpalT38MediaStream(
-      OpalConnection & conn,
+      OpalFaxConnection & conn,
       const OpalMediaFormat & mediaFormat, ///<  Media format for stream
       unsigned sessionID, 
-      PBoolean isSource ,                      ///<  Is a source stream
+      bool isSource,                       ///<  Is a source stream
       const PString & token,               ///<  token used to match incoming/outgoing streams
       const PString & filename,            ///<  filename
-      PBoolean receive,
+      bool receive,
       const PString & stationId
     );
 
     PString GetSpanDSPCommandLine(OpalFaxCallInfo &);
 
-    PBoolean ReadPacket(RTP_DataFrame & packet);
-    PBoolean WritePacket(RTP_DataFrame & packet);
+    virtual PBoolean ReadPacket(RTP_DataFrame & packet);
+    virtual PBoolean WritePacket(RTP_DataFrame & packet);
 
     RTP_DataFrameList queuedFrames;
 };
@@ -222,8 +217,9 @@ class OpalFaxEndPoint : public OpalEndPoint
     /**Create a new endpoint.
      */
     OpalFaxEndPoint(
-      OpalManager & manager,      ///<  Manager of all endpoints.
-      const char * prefix = "fax" ///<  Prefix for URL style address strings
+      OpalManager & manager,        ///<  Manager of all endpoints.
+      const char * g711Prefix = "fax", ///<  Prefix for URL style address strings
+      const char * t38Prefix = "t38"  ///<  Prefix for URL style address strings
     );
 
     /**Destroy endpoint.
@@ -243,10 +239,11 @@ class OpalFaxEndPoint : public OpalEndPoint
       */
     virtual OpalFaxConnection * CreateConnection(
       OpalCall & call,          ///< Owner of connection
+      void * userData,
+      OpalConnection::StringOptions * stringOptions,
       const PString & filename, ///< filename to send/receive
-      PBoolean receive,
-      void * userData = NULL,   ///< Arbitrary data to pass to connection
-      OpalConnection::StringOptions * stringOptions = NULL
+      bool receive,
+      bool t38
     );
 
     /**Get the data formats this endpoint is capable of operating.
@@ -260,24 +257,12 @@ class OpalFaxEndPoint : public OpalEndPoint
       */
     virtual OpalMediaFormatList GetMediaFormats() const;
 
-    virtual PString MakeToken();
-
   /**@name User Interface operations */
     /**Accept the incoming connection.
       */
     virtual void AcceptIncomingConnection(
       const PString & connectionToken ///<  Token of connection to accept call
     );
-
-    /**Call back when patching a media stream.
-       This function is called when a connection has created a new media
-       patch between two streams.
-      */
-    virtual void OnPatchMediaStream(
-      const OpalFaxConnection & connection, ///<  Connection having new patch
-      PBoolean isSource,                         ///<  Source patch
-      OpalMediaPatch & patch                 ///<  New patch
-    );
   //@}
 
   /**@name Member variable access */
@@ -300,13 +285,17 @@ class OpalFaxEndPoint : public OpalEndPoint
     void SetDefaultDirectory(
       const PString & dir    /// New directory for fax reception
     ) { m_defaultDirectory = dir; }
+
+    const PString & GetT38Prefix() const { return m_t38Prefix; }
   //@}
 
   protected:
+    PString    m_t38Prefix;
     PFilePath  m_spanDSP;
     PDirectory m_defaultDirectory;
 };
 
+
 ///////////////////////////////////////////////////////////////////////////////
 
 /** Fax Connection
@@ -320,11 +309,11 @@ class OpalFaxConnection : public OpalConnection
     /**Create a new endpoint.
      */
     OpalFaxConnection(
-      OpalCall & call,                 ///<   Owner calll for connection
-      OpalFaxEndPoint & endpoint,      ///<   Owner endpoint for connection
-      const PString & filename,        ///<   filename to send/receive
-      PBoolean receive,                    ///<   PTrue if receiving a fax
-      const PString & _token,           ///<  token for connection
+      OpalCall & call,                 ///< Owner calll for connection
+      OpalFaxEndPoint & endpoint,      ///< Owner endpoint for connection
+      const PString & filename,        ///< filename to send/receive
+      bool receive,                    ///< true if receiving a fax
+      const PString & _token,          ///< token for connection
       OpalConnection::StringOptions * stringOptions = NULL
     );
 
@@ -377,19 +366,9 @@ class OpalFaxConnection : public OpalConnection
       */
     virtual OpalMediaFormatList GetMediaFormats() const;
 
-    OpalMediaStream * CreateMediaStream(const OpalMediaFormat & mediaFormat, unsigned sessionID, PBoolean isSource);
-
-    /**Call back when patching a media stream.
-       This function is called when a connection has created a new media
-       patch between two streams.
-       Add the echo canceler patch and call the endpoint function of
-       the same name.
-       Add a PCM silence detector filter.
-      */
-    virtual void OnPatchMediaStream(
-      PBoolean isSource,
-      OpalMediaPatch & patch    ///<  New patch
-    );
+    virtual void ApplyStringOptions(OpalConnection::StringOptions & stringOptions);
+    virtual OpalMediaStream * CreateMediaStream(const OpalMediaFormat & mediaFormat, unsigned sessionID, PBoolean isSource);
+    virtual void AdjustMediaFormats(OpalMediaFormatList & mediaFormats) const;
 
   /**@name New operations */
   //@{
@@ -397,45 +376,19 @@ class OpalFaxConnection : public OpalConnection
       */
     virtual void AcceptIncoming();
 
+    /** Check for no more I/O which means fax stopped.
+      */
+    void CheckFaxStopped();
   //@}
 
-    void AdjustMediaFormats(OpalMediaFormatList & mediaFormats) const;
-
   protected:
-    OpalFaxEndPoint & endpoint;
-    PString filename;
-    PBoolean receive;
-    PBoolean forceFaxAudio;
-    PString stationId;
-};
+    PDECLARE_NOTIFIER(PTimer,  OpalFaxConnection, OnFaxStoppedTimeout);
 
-/////////////////////////////////////////////////////////////////////////////////
-
-class OpalT38Connection;
-
-/** T.38 Endpoint
-    This class represents connection that can take a standard group 3 fax
-    TIFF file and produce T.38 packets. This may be directed to another endpoint
-    such as OpalSIPEndpoint or OpalH323EndPoint which can send the TIFF file to
-    "network" fax.
-
-    Relies on the presence of spandsp to do the hard work.
- */
-class OpalT38EndPoint : public OpalFaxEndPoint
-{
-  PCLASSINFO(OpalT38EndPoint, OpalFaxEndPoint);
-  public:
-  /**@name Construction */
-  //@{
-    /**Create a new endpoint.
-     */
-    OpalT38EndPoint(
-      OpalManager & manager,      ///<  Manager of all endpoints.
-      const char * prefix = "t38" ///<  Prefix for URL style address strings
-    );
-    OpalMediaFormatList GetMediaFormats() const;
-    PString MakeToken();
-    virtual OpalFaxConnection * CreateConnection(OpalCall & call, const PString & filename, PBoolean receive, void * /*userData*/, OpalConnection::StringOptions * stringOptions);
+    OpalFaxEndPoint & m_endpoint;
+    PString           m_filename;
+    bool              m_receive;
+    PString           m_stationId;
+    PTimer            m_faxStopped;
 };
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -452,26 +405,26 @@ class OpalT38Connection : public OpalFaxConnection
      */
     OpalT38Connection(
       OpalCall & call,                 ///<  Owner calll for connection
-      OpalT38EndPoint & endpoint,      ///<  Owner endpoint for connection
+      OpalFaxEndPoint & endpoint,      ///<  Owner endpoint for connection
       const PString & filename,        ///<  filename to send/receive
-      PBoolean receive,                ///<  true if receiving fax
-      const PString & _token,          ///<  token for connection
+      bool receive,                    ///<  true if receiving fax
+      const PString & token,           ///<  token for connection
       OpalConnection::StringOptions * stringOptions = NULL
     );
 
-    ~OpalT38Connection();
-
-    void OnPatchMediaStream(PBoolean isSource, OpalMediaPatch & patch);
-    OpalMediaStream * CreateMediaStream(const OpalMediaFormat & mediaFormat, unsigned sessionID, PBoolean isSource);
-    OpalMediaFormatList GetMediaFormats() const;
+    virtual PString GetPrefixName() const { return m_endpoint.GetT38Prefix(); }
+    virtual void ApplyStringOptions(OpalConnection::StringOptions & stringOptions);
+    virtual void OnEstablished();
+    virtual OpalMediaStream * CreateMediaStream(const OpalMediaFormat & mediaFormat, unsigned sessionID, PBoolean isSource);
+    virtual void OnClosedMediaStream(const OpalMediaStream & stream);
+    virtual OpalMediaFormatList GetMediaFormats() const;
 
     // triggers into fax mode
-    enum {
-      T38Mode_Wait         = 0,   // Do nothing and wait for the other end to send a reinvite
-      T38Mode_InBandCED    = 1,   // If receiving, send CED tone in-band. If sending, trigger mode change if CED detected in-band
-      T38Mode_NSECED       = 2,   // If receiving, send CED as RFC2833. If sending, trigger mode change if RFC2833 CED received
-      T38Mode_Timeout      = 4,   // Trigger mode change if has not occurred after 5 seconds
-      T38Mode_Auto         = 7    // All of the above :)
+    enum Mode {
+      Mode_Wait,      // Do nothing and wait for the other end to send to switch to T.38 mode
+      Mode_Timeout,   // Switch to T.38 mode a short time after call is established
+      Mode_UserInput, // Send CNG/CED as UserInput (e.g. RFC2833).
+      Mode_InBand     // Send CNG/CED tone in-band (not yet implemented!)
     };
 
     virtual PBoolean SendUserInputTone(
@@ -479,22 +432,19 @@ class OpalT38Connection : public OpalFaxConnection
       unsigned duration
     );
 
-    PDECLARE_NOTIFIER(PTimer, OpalT38Connection, OnFaxChangeTimeout);
-
   protected:
+    PDECLARE_NOTIFIER(PTimer,  OpalT38Connection, OnSendCNGCED);
+    PDECLARE_NOTIFIER(PTimer,  OpalT38Connection, OnFaxChangeTimeout);
+    PDECLARE_NOTIFIER(PThread, OpalT38Connection, OpenFaxStreams);
     void RequestFaxMode(bool fax);
-    void InFaxMode(bool fax);
 
-    unsigned t38WaitMode;
+    Mode m_syncMode;
 
-    PMutex modeMutex;
-    bool currentMode, newMode;  // false if audio, true if fax
-    bool modeChangeTriggered;
-    PTimer faxTimer;
-    bool faxStartup;
+    bool   m_faxMode;  // false if audio, true if fax
+    PTimer m_faxTimer;
 };
 
 
 #endif // OPAL_FAX
 
-#endif // __OPAL_T38PROTO_H
+#endif // OPAL_T38_T38PROTO_H
