@@ -19,9 +19,9 @@
  *
  * Contributor(s): Equivalence Pty. Ltd.
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 21293 $
+ * $Author: rjongbloed $
+ * $Date: 2008-10-12 23:24:41 +0000 (Sun, 12 Oct 2008) $
  */
 
 #include "precompile.h"
@@ -84,8 +84,8 @@ void CallGen::Main()
   
   if (args.GetCount() == 0 && !args.HasOption('l')) {
     cout << "Usage:\n"
-            "  callgen [options] -l\n"
-            "  callgen [options] destination [ destination ... ]\n"
+            "  " << GetFile().GetTitle() << " [options] -l\n"
+            "  " << GetFile().GetTitle() << " [options] destination [ destination ... ]\n"
             "where options:\n"
             "  -l                   Passive/listening mode.\n"
             "  -m --max num         Maximum number of simultaneous calls\n"
