@@ -23,14 +23,16 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 21762 $
+ * $Author: rjongbloed $
+ * $Date: 2008-12-10 07:19:14 +0000 (Wed, 10 Dec 2008) $
  */
 
 #include "precompile.h"
 #include "main.h"
 
+#include <opal/patch.h>
+
 
 PCREATE_PROCESS(CodecTest);
 
@@ -51,26 +53,30 @@ void CodecTest::Main()
 
   args.Parse("b-bit-rate:"
              "c-crop."
+             "C-rate-control."
              "D-display-device:"
+             "-display-driver:"
              "F-audio-frames:"
              "G-grab-device:"
+             "-grab-driver:"
+             "-grab-format:"
+             "-grab-channel:"
              "h-help."
              "m-suppress-marker."
              "M-force-marker."
              "O-option:"
              "p-payload-size:"
              "P-play-device:"
+             "-play-driver:"
+             "-play-buffers:"
              "r-frame-rate:"
+             "R-record-device:"
+             "-record-driver:"
              "s-frame-size:"
              "S-simultaneous:"
-             "-record-driver:"
-             "-play-buffers:"
-             "R-record-device:"
-             "-play-driver:"
-             "-grab-driver:"
-             "-grab-format:"
-             "-grab-channel:"
-             "-display-driver:"
+             "-count:"
+             "-noprompt."
+             "-snr."
 #if PTRACING
              "o-output:"             "-no-output."
              "t-trace."              "-no-trace."
@@ -84,7 +90,7 @@ void CodecTest::Main()
 #endif
 
   if (args.HasOption('h') || args.GetCount() == 0) {
-    PError << "usage: codectest [ options ] fmtname [ fmtname ]\n"
+    PError << "usage: " << GetFile().GetTitle() << " [ options ] fmtname [ fmtname ]\n"
               "  where fmtname is the Media Format Name for the codec(s) to test, up to two\n"
               "  formats (one audio and one video) may be specified.\n"
               "\n"
@@ -106,17 +112,23 @@ void CodecTest::Main()
               "  -r --frame-rate size    : video frame rate (frames/second)\n"
               "  -b --bit-rate size      : video bit rate (bits/second)\n"
               "  -O --option opt=val     : set media format option to value\n"
+              "  -S --single-step        : video single frame at a time mode\n"
               "  -c --crop               : crop rather than scale if resizing\n"
               "  -m --suppress-marker    : suppress marker bits to decoder"
               "  -M --force-marker       : force marker bits to decoder"
               "  -p --payload-size sz    : Set size of maximum RTP payload for encoded data\n"
               "  -S --simultanoues n     : Number of simultaneous encode/decode threads\n"
+              "  -T --statistics         : output statistics files\n"
+              "  -C --rate-control       : enable rate control\n"
+              "  --count n               : set number of frames to transcode\n"
+              "  --noprompt              : do not prompt for commands, i.e. exit when input closes\n"
+              "  --snr                   : calculate signal-to-noise ratio between input and output"
 #if PTRACING
               "  -o or --output file     : file name for output of log messages\n"       
               "  -t or --trace           : degree of verbosity in error log (more times for more detail)\n"     
 #endif
               "\n"
-              "e.g. ./codectest --grab-device fake --grab-channel 2 GSM-AMR H.264\n\n";
+              "e.g. " << GetFile().GetTitle() << " --grab-device fake --grab-channel 2 GSM-AMR H.264\n\n";
     return;
   }
 
@@ -156,6 +168,7 @@ void CodecTest::Main()
   if (!audio.Initialise(args) || !video.Initialise(args))
     return;
 
+#if 0
   if (video.encoder == NULL && audio.encoder == NULL) {
     cout << "Could not identify media formats ";
     for (int i = 0; i < args.GetCount(); ++i)
@@ -163,73 +176,90 @@ void CodecTest::Main()
     cout << endl;
     return;
   }
+#endif
 
   audio.Resume();
   video.Resume();
 
   // command line
-  for (;;) {
+  if (args.HasOption("noprompt")) {
+    cout << "Waiting for finish" << endl;
+    audio.WaitForTermination();
+    video.WaitForTermination();
+  }
+  else {
+    for (;;) {
 
-    // display the prompt
-    cout << "codectest> " << flush;
-    PCaselessString cmd;
-    cin >> cmd;
+      // display the prompt
+      cout << "codectest> " << flush;
+      PCaselessString cmd;
+      cin >> cmd;
 
-    if (cmd == "q" || cmd == "x" || cmd == "quit" || cmd == "exit")
-      break;
+      if (cmd == "q" || cmd == "x" || cmd == "quit" || cmd == "exit")
+        break;
 
-    if (cmd == "vfu") {
-      if (video.encoder == NULL)
-        cout << "\nNo video encoder running!" << endl;
-      else
-        video.encoder->ExecuteCommand(OpalVideoUpdatePicture());
-      continue;
-    }
+      if (cmd.NumCompare("n") == EqualTo) {
+        int steps = cmd.Mid(1).AsUnsigned();
+        do {
+          video.frameWait.Signal();
+        } while (--steps > 0);
+        continue;
+      }
 
-    if (cmd == "fg") {
-      if (video.grabber == NULL)
-        cout << "\nNo video grabber running!" << endl;
-      else if (!video.grabber->SetVFlipState(!video.grabber->GetVFlipState()))
-        cout << "\nCould not toggle Vflip state of video grabber device" << endl;
-      continue;
-    }
+      if (cmd == "vfu") {
+        if (video.encoder == NULL)
+          cout << "\nNo video encoder running!" << endl;
+        else
+          video.encoder->ExecuteCommand(OpalVideoUpdatePicture());
+        continue;
+      }
 
-    if (cmd == "fd") {
-      if (video.display == NULL)
-        cout << "\nNo video display running!" << endl;
-      else if (!video.display->SetVFlipState(!video.display->GetVFlipState()))
-        cout << "\nCould not toggle Vflip state of video display device" << endl;
-      continue;
-    }
+      if (cmd == "fg") {
+        if (video.grabber == NULL)
+          cout << "\nNo video grabber running!" << endl;
+        else if (!video.grabber->SetVFlipState(!video.grabber->GetVFlipState()))
+          cout << "\nCould not toggle Vflip state of video grabber device" << endl;
+        continue;
+      }
 
-    unsigned width, height;
-    if (PVideoFrameInfo::ParseSize(cmd, width, height)) {
-      video.pause.Signal();
-      if (video.grabber == NULL)
-        cout << "\nNo video grabber running!" << endl;
-      else if (!video.grabber->SetFrameSizeConverter(width, height))
-        cout << "Video grabber device could not be set to size " << width << 'x' << height << endl;
-      if (video.display == NULL)
-        cout << "\nNo video display running!" << endl;
-      else if (!video.display->SetFrameSizeConverter(width, height))
-        cout << "Video display device could not be set to size " << width << 'x' << height << endl;
-      video.resume.Signal();
-      continue;
-    }
+      if (cmd == "fd") {
+        if (video.display == NULL)
+          cout << "\nNo video display running!" << endl;
+        else if (!video.display->SetVFlipState(!video.display->GetVFlipState()))
+          cout << "\nCould not toggle Vflip state of video display device" << endl;
+        continue;
+      }
 
-    cout << "Select:\n"
-            "  vfu    : Video Fast Update (force I-Frame)\n"
-            "  fg     : Flip video grabber top to bottom\n"
-            "  fd     : Flip video display top to bottom\n"
-            "  qcif   : Set size of grab & display to qcif\n"
-            "  cif    : Set size of grab & display to cif\n"
-            "  WxH    : Set size of grab & display W by H\n"
-            "  Q or X : Exit program\n" << endl;
-  } // end for
+      unsigned width, height;
+      if (PVideoFrameInfo::ParseSize(cmd, width, height)) {
+        video.pause.Signal();
+        if (video.grabber == NULL)
+          cout << "\nNo video grabber running!" << endl;
+        else if (!video.grabber->SetFrameSizeConverter(width, height))
+          cout << "Video grabber device could not be set to size " << width << 'x' << height << endl;
+        if (video.display == NULL)
+          cout << "\nNo video display running!" << endl;
+        else if (!video.display->SetFrameSizeConverter(width, height))
+          cout << "Video display device could not be set to size " << width << 'x' << height << endl;
+        video.resume.Signal();
+        continue;
+      }
 
-  cout << "Exiting." << endl;
-  audio.Stop();
-  video.Stop();
+      cout << "Select:\n"
+              "  vfu    : Video Fast Update (force I-Frame)\n"
+              "  fg     : Flip video grabber top to bottom\n"
+              "  fd     : Flip video display top to bottom\n"
+              "  qcif   : Set size of grab & display to qcif\n"
+              "  cif    : Set size of grab & display to cif\n"
+              "  WxH    : Set size of grab & display W by H\n"
+              "  N      : Next video frame in sinlge step mode\n"
+              "  Q or X : Exit program\n" << endl;
+    } // end for
+
+    cout << "Exiting." << endl;
+    audio.Stop();
+    video.Stop();
+  }
 }
 
 
@@ -240,6 +270,13 @@ int TranscoderThread::InitialiseCodec(PArgList & args, const OpalMediaFormat & r
   else if (args.HasOption('M'))
     markerHandling = ForceMarkers;
 
+  framesToTranscode = -1;
+  PString s = args.GetOptionString("count");
+  if (!s.IsEmpty())
+    framesToTranscode = s.AsInteger();
+
+  calcSNR = args.HasOption("snr");
+
   for (PINDEX i = 0; i < args.GetCount(); i++) {
     OpalMediaFormat mediaFormat = args[i];
     if (mediaFormat.IsEmpty()) {
@@ -304,6 +341,27 @@ void SetOptions(PArgList & args, OpalMediaFormat & mediaFormat)
   mediaFormat.ToCustomisedOptions();
 }
 
+static int udiff(unsigned int const subtrahend, unsigned int const subtractor) 
+{
+  return subtrahend - subtractor;
+}
+
+
+static double square(double const arg) 
+{
+  return(arg*arg);
+}
+
+static double CalcSNR(const BYTE * src1, const BYTE * src2, PINDEX dataLen)
+{
+  double diff2 = 0.0;
+  for (PINDEX i = 0; i < dataLen; ++i) 
+    diff2 += square(udiff(*src1++, *src2++));
+
+  double const snr = diff2 / dataLen / 255;
+
+  return snr;
+}
 
 bool AudioThread::Initialise(PArgList & args)
 {
@@ -315,7 +373,7 @@ bool AudioThread::Initialise(PArgList & args)
   }
 
   readSize = encoder != NULL ? encoder->GetOptimalDataFrameSize(TRUE) : 480;
-  OpalMediaFormat mediaFormat = encoder != NULL ? encoder->GetOutputFormat() : OpalYUV420P;
+  OpalMediaFormat mediaFormat = encoder != NULL ? encoder->GetOutputFormat() : OpalPCM16;
 
   cout << "Audio media format set to " << mediaFormat << endl;
 
@@ -355,7 +413,6 @@ bool AudioThread::Initialise(PArgList & args)
 
   cout << "opened and initialised." << endl;
 
-
   // Audio player
   driverName = args.GetOptionString("play-driver");
   deviceName = args.GetOptionString("play-device");
@@ -390,7 +447,9 @@ bool AudioThread::Initialise(PArgList & args)
     return false;
   }
 
-  if (encoder != NULL) {
+  if (encoder == NULL) 
+    frameTime = mediaFormat.GetFrameTime();
+  else {
     OpalMediaFormat mediaFormat = encoder->GetOutputFormat();
     SetOptions(args, mediaFormat);
     encoder->UpdateMediaFormats(OpalMediaFormat(), mediaFormat);
@@ -474,7 +533,6 @@ bool VideoThread::Initialise(PArgList & args)
   cout << "Grabber channel set to " << grabber->GetChannel() << endl;
 
   
-  unsigned frameRate;
   if (args.HasOption("frame-rate"))
     frameRate = args.GetOptionString("frame-rate").AsUnsigned();
   else
@@ -489,7 +547,6 @@ bool VideoThread::Initialise(PArgList & args)
 
   cout << "Grabber frame rate set to " << grabber->GetFrameRate() << endl;
 
-
   // Video display
   driverName = args.GetOptionString("display-driver");
   deviceName = args.GetOptionString("display-device");
@@ -545,7 +602,6 @@ bool VideoThread::Initialise(PArgList & args)
 
   cout << "Display frame size set to " << display->GetFrameWidth() << 'x' << display->GetFrameHeight() << endl;
 
-
   if (!grabber->SetColourFormatConverter("YUV420P") ) {
     cerr << "Video grabber device could not be set to colour format YUV420P" << endl;
     return false;
@@ -594,14 +650,33 @@ bool VideoThread::Initialise(PArgList & args)
   }
   cout << "Target bit rate set to " << mediaFormat.GetOptionInteger(OpalVideoFormat::TargetBitRateOption()) << " bps" << endl;
 
+  if (args.HasOption('T')) {
+    frameFn  = "frame_stats.csv";
+    packetFn = "packet_stats.csv";
+  }
+
+  if (args.HasOption('C')) {
+    rcEnable = true;
+    unsigned rate = mediaFormat.GetOptionInteger(OpalVideoFormat::TargetBitRateOption());
+    rateController.Open(rate);
+    cout << "Video rate controller enabled for bit rate " << rate << " bps" << endl;
+  }
+
   SetOptions(args, mediaFormat);
 
-  if (encoder != NULL) {
+  if (encoder == NULL) 
+    frameTime = mediaFormat.GetFrameTime();
+  else {
     encoder->UpdateMediaFormats(OpalMediaFormat(), mediaFormat);
     if (args.HasOption('p'))
       encoder->SetMaxOutputSize(args.GetOptionString('p').AsUnsigned());
   }
 
+  singleStep = args.HasOption('S');
+
+  sumYSNR = sumCbSNR = sumCrSNR = 0.0;
+  snrCount = 0;
+
   return true;
 }
 
@@ -627,22 +702,110 @@ void VideoThread::Main()
   grabber->Start();
   display->Start();
 
+  //decoder->SetCommandNotifier(PCREATE_NOTIFIER(OnTranscoderCommand));
+
+  forceIFrame = false;
   TranscoderThread::Main();
 }
 
+void TranscoderThread::OnTranscoderCommand(OpalMediaCommand & cmd, INT)
+{
+  if (PIsDescendant(&cmd, OpalVideoUpdatePicture)) {
+    cout << "decoder lost sync" << endl;
+    forceIFrame = true;
+    ((OpalVideoTranscoder *)encoder)->ForceIFrame();
+  }
+  else {
+    cout << "unknown decoder command " << cmd.GetName() << endl;
+  }
+}
+
+void VideoThread::CalcSNR(const RTP_DataFrame & src, const RTP_DataFrame & dst)
+{
+  if (src.GetPayloadSize() < sizeof(OpalVideoTranscoder::FrameHeader) || dst.GetPayloadSize() < sizeof(OpalVideoTranscoder::FrameHeader))
+    return;
+
+  const BYTE * src1 = src.GetPayloadPtr();
+  const BYTE * src2 = dst.GetPayloadPtr();
+
+  const OpalVideoTranscoder::FrameHeader * hdr1 = (OpalVideoTranscoder::FrameHeader *)src1;
+  const OpalVideoTranscoder::FrameHeader * hdr2 = (OpalVideoTranscoder::FrameHeader *)src2;
+
+  if (hdr1->height != hdr2->height || hdr1->width != hdr2->width)
+    return;
+
+  if (hdr1->height != snrHeight || hdr1->width != snrWidth) {
+    sumYSNR = sumCbSNR = sumCrSNR = 0.0;
+    snrCount = 0;
+    snrHeight = hdr1->height;
+    snrWidth  = hdr1->width;
+  }
+
+  unsigned size = snrHeight * snrWidth;
+
+  int tsize = sizeof(OpalVideoTranscoder::FrameHeader) + size*3/2;
+
+  if (src.GetPayloadSize() < tsize || dst.GetPayloadSize() < tsize)
+    return;
+
+  src1 += sizeof(OpalVideoTranscoder::FrameHeader);
+  src2 += sizeof(OpalVideoTranscoder::FrameHeader);
+
+  sumYSNR  = ::CalcSNR(src1, src2, size);
+  src1 += size;
+  src2 += size;
+
+  size = size / 4;
+  sumCbSNR = ::CalcSNR(src1, src2, size);
+  src1 += size;
+  src2 += size;
+
+  sumCrSNR = ::CalcSNR(src1, src2, size);
+
+  ++snrCount;
+}
+
+void VideoThread::ReportSNR()
+{
+  /* The PSNR is the mean of the sum of squares of the differences,
+     normalized to the range 0..1
+  */
+  double const yPsnr = sumYSNR / snrCount;
+
+  if (yPsnr <= 1e-9)
+    cout << "Y  color component identical" << endl;
+  else
+    printf("Y  color component avg SNR : %.2f dB\n", 10 * log10(1/yPsnr));
+
+  double const cbPsnr = sumCbSNR / snrCount;
+
+  if (cbPsnr <= 1e-9)
+    cout << "Cb color component identical" << endl;
+  else
+    printf("Cb color component avg SNR : %.2f dB\n", 10 * log10(1/cbPsnr));
+
+  double const crPsnr = sumCrSNR / snrCount;
+
+  if (crPsnr <= 1e-9)
+    cout << "Cr color component identical" << endl;
+  else
+    printf("Cr color component avg SNR : %.2f dB\n", 10 * log10(1/crPsnr));
+}
+
 
 void TranscoderThread::Main()
 {
   PUInt64 byteCount = 0;
   PUInt64 frameCount = 0;
   PUInt64 packetCount = 0;
+  WORD sequenceNumber = 0;
   bool oldSrcState = true;
   bool oldOutState = true;
   bool oldEncState = true;
   bool oldDecState = true;
 
   PTimeInterval startTick = PTimer::Tick();
-  while (running) {
+  while (running && framesToTranscode < 0 || (framesToTranscode-- > 0)) {
 
     RTP_DataFrame srcFrame;
     bool state = Read(srcFrame);
@@ -652,10 +815,15 @@ void TranscoderThread::Main()
     }
 
     srcFrame.SetTimestamp(timestamp);
-    timestamp += encoder->GetOutputFormat().GetFrameTime();
+    timestamp += frameTime;
 
+    if (rcEnable && rateController.SkipFrame()) {
+      cerr << "Rate controller forced frame skip" << endl;
+      continue;
+    }
+      
     RTP_DataFrameList encFrames;
-    if (encoder == NULL)
+    if (encoder == NULL) 
       encFrames.Append(new RTP_DataFrame(srcFrame)); 
     else {
       state = encoder->ConvertFrames(srcFrame, encFrames);
@@ -666,6 +834,12 @@ void TranscoderThread::Main()
       }
     }
 
+    for (PINDEX i = 0; i < encFrames.GetSize(); i++) {
+      encFrames[i].SetSequenceNumber(++sequenceNumber);
+    }
+
+    unsigned long frameSize = 0;
+    unsigned long framePacketCount = 0;
     for (PINDEX i = 0; i < encFrames.GetSize(); i++) {
       RTP_DataFrameList outFrames;
       if (encoder == NULL)
@@ -681,13 +855,20 @@ void TranscoderThread::Main()
         }
 
         state = decoder->ConvertFrames(encFrames[i], outFrames);
+
         if (oldDecState != state) {
           oldDecState = state;
           cerr << "Decoder " << (state ? "restor" : "fail") << "ed at packet " << packetCount << endl;
           continue;
         }
+        UpdateStats(encFrames[i]);
       }
+
       for (PINDEX j = 0; j < outFrames.GetSize(); j++) {
+
+        if (calcSNR) 
+          CalcSNR(srcFrame, outFrames[j]);
+
         state = Write(outFrames[j]);
         if (oldOutState != state) {
           oldOutState = state;
@@ -695,9 +876,15 @@ void TranscoderThread::Main()
         }
       }
       byteCount += encFrames[i].GetPayloadSize();
+      frameSize += encFrames[i].GetPayloadSize() + encFrames[i].GetHeaderSize();
+      framePacketCount++;
       packetCount++;
     }
 
+
+    if (rcEnable)
+      rateController.AddFrame(frameSize, framePacketCount);
+
     if (pause.Wait(0)) {
       pause.Acknowledge();
       resume.Wait();
@@ -728,6 +915,9 @@ void TranscoderThread::Main()
   else if (bitRate < 10000000000ULL)
     cout << bitRate/1000000.0 << " M";
   cout << "bits/s." << endl;
+
+  if (calcSNR) 
+    ReportSNR();
 }
 
 
@@ -744,8 +934,18 @@ bool AudioThread::Write(const RTP_DataFrame & frame)
 }
 
 
+void AudioThread::Stop()
+{
+  running = false;
+  WaitForTermination();
+}
+
+
 bool VideoThread::Read(RTP_DataFrame & data)
 {
+  if (singleStep)
+    frameWait.Wait();
+
   data.SetPayloadSize(grabber->GetMaxFrameBytes()+sizeof(OpalVideoTranscoder::FrameHeader));
   data.SetMarker(TRUE);
 
@@ -770,4 +970,64 @@ bool VideoThread::Write(const RTP_DataFrame & data)
 }
 
 
+void VideoThread::Stop()
+{
+  running = false;
+  frameWait.Signal();
+  WaitForTermination();
+}
+
+void VideoThread::InitStats()
+{
+  packetCount= 0;
+  frameCount = 0;
+  frameBytes = 0;
+  totalFrameBytes = 0;
+}
+
+void VideoThread::UpdateStats(const RTP_DataFrame & frame)
+{
+  if (!packetFn.IsEmpty()) {
+    if (packetStatFile.Open(packetFn, PFile::WriteOnly))
+      packetStatFile << "PacketNum,PacketSize\n";
+    packetFn.MakeEmpty();
+  }
+
+  packetCount++;
+  frameBytes      += frame.GetPayloadSize();
+  totalFrameBytes += frame.GetPayloadSize();
+
+  if (packetStatFile.IsOpen()) 
+    packetStatFile << packetCount << ','
+                   << frame.GetPayloadSize() << endl;
+
+  if (frame.GetMarker()) 
+    UpdateFrameStats();
+}
+
+PInt64 BpsTokbps(PInt64 Bps)
+{
+  return ((Bps * 8) + 500) / 1000;
+}
+
+void VideoThread::UpdateFrameStats()
+{
+  if (!frameFn.IsEmpty()) {
+    if (frameStatFile.Open(frameFn, PFile::WriteOnly))
+      frameStatFile << "FrameNum,FrameBits,FrameKBPS,TotalKBPS\n";
+    frameFn.MakeEmpty();
+  }
+
+  frameCount++;
+
+  if (frameStatFile.IsOpen()) {
+    frameStatFile << frameCount << ','
+                  << (frameBytes * 8) << ','
+                  << BpsTokbps(frameBytes * frameRate) << ','
+                  << BpsTokbps((totalFrameBytes * frameRate) / frameCount) << endl;
+  }
+
+  frameBytes = 0;
+}
+
 // End of File ///////////////////////////////////////////////////////////////
