@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 21655 $
+ * $Author: rjongbloed $
+ * $Date: 2008-11-30 05:16:47 +0000 (Sun, 30 Nov 2008) $
  */
 
 #include <ptlib.h>
@@ -348,11 +348,46 @@ void H323GetApplicationInfo(OpalProductInfo & info, const H225_VendorIdentifier
 
 ///////////////////////////////////////////////////////////////////////////////
 
+void H323SetRTPPacketization(H245_ArrayOf_RTPPayloadType & rtpPacketizations,
+                             PINDEX & rtpPacketizationCount,
+                             const OpalMediaFormat & mediaFormat,
+                             RTP_DataFrame::PayloadTypes payloadType)
+{
+  PString mediaPacketization = mediaFormat.GetOptionString(OpalMediaFormat::MediaPacketizationsOption(),
+                               mediaFormat.GetOptionString(OpalMediaFormat::MediaPacketizationOption()));
+  if (mediaPacketization.IsEmpty())
+    return;
+  
+  // Special case handling by Product Id
+  PString h323ProductId = mediaFormat.GetOptionString("h323ProductId");
+  if (h323ProductId == "NetMeeting")
+    return;
+
+  PStringArray packetizationStrings = mediaPacketization.Tokenise(",");
+  for (PINDEX i = 0; i < packetizationStrings.GetSize(); i++) {
+    PString & packetizationString = packetizationStrings[i];
+    
+    rtpPacketizations.SetSize(rtpPacketizationCount+1);
+    if (H323SetRTPPacketization(rtpPacketizations[rtpPacketizationCount], packetizationString, mediaFormat, payloadType)) {
+      
+      // Check if already in list
+      PINDEX test;
+      for (test = 0; test < rtpPacketizationCount; test++) {
+        if (rtpPacketizations[test] == rtpPacketizations[rtpPacketizationCount])
+          break;
+      }
+      if (test == rtpPacketizationCount)
+        rtpPacketizationCount++;
+    }
+  }
+}
+
 bool H323SetRTPPacketization(H245_RTPPayloadType & rtpPacketization,
                              const OpalMediaFormat & mediaFormat,
                              RTP_DataFrame::PayloadTypes payloadType)
 {
-  PString mediaPacketization = mediaFormat.GetOptionString(OpalMediaFormat::MediaPacketizationOption());
+  PString mediaPacketization = mediaFormat.GetOptionString(OpalMediaFormat::MediaPacketizationsOption(),
+                               mediaFormat.GetOptionString(OpalMediaFormat::MediaPacketizationOption()));
   if (mediaPacketization.IsEmpty())
     return PFalse;
 
@@ -360,14 +395,25 @@ bool H323SetRTPPacketization(H245_RTPPayloadType & rtpPacketization,
   PString h323ProductId = mediaFormat.GetOptionString("h323ProductId");
   if (h323ProductId == "NetMeeting")
     return PFalse;
+  
+  PStringArray packetizationStrings = mediaPacketization.Tokenise(",");
+  
+  // Only use the first packetization (= highest priority)
+  return H323SetRTPPacketization(rtpPacketization, packetizationStrings[0], mediaFormat, payloadType);
+}
 
-  if (mediaPacketization.NumCompare("RFC") == PObject::EqualTo) {
+bool H323SetRTPPacketization(H245_RTPPayloadType & rtpPacketization,
+                             const PString & packetizationString,
+                             const OpalMediaFormat & mediaFormat,
+                             RTP_DataFrame::PayloadTypes payloadType)
+{
+  if (packetizationString.NumCompare("RFC") == PObject::EqualTo) {
     rtpPacketization.m_payloadDescriptor.SetTag(H245_RTPPayloadType_payloadDescriptor::e_rfc_number);
-    ((PASN_Integer &)rtpPacketization.m_payloadDescriptor) = mediaPacketization.Mid(3).AsUnsigned();
-  }
-  else if (mediaPacketization.FindSpan("0123456789.") == P_MAX_INDEX) {
+    ((PASN_Integer &)rtpPacketization.m_payloadDescriptor) = packetizationString.Mid(3).AsUnsigned();
+  } 
+  else if (packetizationString.FindSpan("0123456789.") == P_MAX_INDEX) {
     rtpPacketization.m_payloadDescriptor.SetTag(H245_RTPPayloadType_payloadDescriptor::e_oid);
-    ((PASN_ObjectId &)rtpPacketization.m_payloadDescriptor) = mediaPacketization;
+    ((PASN_ObjectId &)rtpPacketization.m_payloadDescriptor) = packetizationString;
   }
   else {
     rtpPacketization.m_payloadDescriptor.SetTag(H245_RTPPayloadType_payloadDescriptor::e_nonStandardIdentifier);
@@ -377,7 +423,7 @@ bool H323SetRTPPacketization(H245_RTPPayloadType & rtpPacketization,
     h221.m_t35CountryCode = (unsigned)OpalProductInfo::Default().t35CountryCode;
     h221.m_t35Extension = (unsigned)OpalProductInfo::Default().t35Extension;
     h221.m_manufacturerCode = (unsigned)OpalProductInfo::Default().manufacturerCode;
-    nonstd.m_data = mediaPacketization;
+    nonstd.m_data = packetizationString;
   }
 
   if (payloadType == RTP_DataFrame::IllegalPayloadType)
@@ -426,7 +472,8 @@ bool H323GetRTPPacketization(OpalMediaFormat & mediaFormat, const H245_RTPPayloa
   if (mediaPacketization.IsEmpty())
     return false;
   
-  mediaFormat.SetOptionString(OpalMediaFormat::MediaPacketizationOption(), mediaPacketization);
+  mediaFormat.SetOptionString(OpalMediaFormat::MediaPacketizationsOption(), mediaPacketization);
+  mediaFormat.SetOptionString(OpalMediaFormat::MediaPacketizationOption(), mediaPacketization.Left(mediaPacketization.Find(',')));
   return true;
 }
 
@@ -526,10 +573,8 @@ H225_Setup_UUIE & H323SignalPDU::BuildSetup(const H323Connection & connection,
 
   setup.IncludeOptionalField(H225_Setup_UUIE::e_sourceAddress);
   {
-    OpalConnection::StringOptions * stringOptions = connection.GetStringOptions();
     PString callingParty;
-    if (stringOptions != NULL)
-      callingParty = (*stringOptions)("Calling-Party-Name");
+    callingParty = connection.GetStringOptions()("Calling-Party-Name");
     if (callingParty.IsEmpty())
       H323SetAliasAddresses(endpoint.GetAliasNames(), setup.m_sourceAddress);
     else
@@ -685,11 +730,11 @@ H225_Information_UUIE & H323SignalPDU::BuildInformation(const H323Connection & c
 
 
 H323Connection::CallEndReason H323TranslateToCallEndReason(Q931::CauseValues cause,
-                                               const H225_ReleaseCompleteReason & reason)
+                                                           unsigned reason)
 {
   switch (cause) {
     case Q931::ErrorInCauseIE :
-      switch (reason.GetTag()) {
+      switch (reason) {
         case H225_ReleaseCompleteReason::e_noBandwidth :
           return H323Connection::EndedByNoBandwidth;
 
@@ -758,8 +803,8 @@ H323Connection::CallEndReason H323TranslateToCallEndReason(Q931::CauseValues cau
 }
 
 
-Q931::CauseValues H323TranslateFromCallEndReason(const H323Connection & connection,
-                                                 H225_ReleaseCompleteReason & reason)
+Q931::CauseValues H323TranslateFromCallEndReason(H323Connection::CallEndReason callEndReason,
+                                                 H225_ReleaseCompleteReason & releaseCompleteReason)
 {
   static int const ReasonCodes[H323Connection::NumCallEndReasons] = {
     Q931::NormalCallClearing,                               /// EndedByLocalUser,         Local endpoint application cleared call
@@ -789,13 +834,13 @@ Q931::CauseValues H323TranslateFromCallEndReason(const H323Connection & connecti
     Q931::NormalUnspecified,                                /// EndedByDurationLimit,     Call cleared due to an enforced duration limit
   };
 
-  int code = ReasonCodes[connection.GetCallEndReason()];
+  int code = ReasonCodes[callEndReason];
   if (code == Q931::UnknownCauseIE)
-    return (Q931::CauseValues)connection.GetCallEndReason();
+    return (Q931::CauseValues)callEndReason;
   if (code >= 0)
     return (Q931::CauseValues)code;
 
-  reason.SetTag(-code);
+  releaseCompleteReason.SetTag(-code);
   return Q931::ErrorInCauseIE;
 }
 
@@ -814,7 +859,7 @@ H225_ReleaseComplete_UUIE &
 
   Q931::CauseValues cause = (Q931::CauseValues)connection.GetQ931Cause();
   if (cause == Q931::ErrorInCauseIE)
-    cause = H323TranslateFromCallEndReason(connection, release.m_reason);
+    cause = H323TranslateFromCallEndReason(connection.GetCallEndReason(), release.m_reason);
   if (cause != Q931::ErrorInCauseIE)
     q931pdu.SetCause(cause);
   else
@@ -1184,14 +1229,12 @@ void H323SignalPDU::SetQ931Fields(const H323Connection & connection,
   PString number;
 
   {
-    OpalConnection::StringOptions * stringOptions = connection.GetStringOptions();
+    const OpalConnection::StringOptions & stringOptions = connection.GetStringOptions();
     PString strLocal, strDisplay;
-    if (stringOptions != NULL) {
-      strLocal   = (*stringOptions)("Calling-Party-Name");
-      strDisplay = (*stringOptions)("Calling-Display-Name");
-      if (strDisplay.IsEmpty())
-        strDisplay = strLocal;
-    }
+    strLocal   = stringOptions("Calling-Party-Name");
+    strDisplay = stringOptions("Calling-Display-Name");
+    if (strDisplay.IsEmpty())
+      strDisplay = strLocal;
 
     if (!strLocal.IsEmpty())
       localName = strLocal;
