@@ -24,16 +24,16 @@
  *
  * The author of this code is Derek J Smithies
  *
- *
- *
- * $Revision: 21004 $
+ * $Revision: 21283 $
  * $Author: rjongbloed $
- * $Date: 2008-09-16 07:08:56 +0000 (Tue, 16 Sep 2008) $
+ * $Date: 2008-10-11 07:10:58 +0000 (Sat, 11 Oct 2008) $
  */
 
 #include <ptlib.h>
 #include <opal/buildopts.h>
 
+#if OPAL_IAX2
+
 #ifdef P_USE_PRAGMA
 #pragma implementation "iedata.h"
 #endif
@@ -124,6 +124,9 @@ void IAX2IeData::PrintOn(ostream & strm) const
        <<   "Frames received Out of Order u32                     " << receivedOoo       << endl;
 }
 
+
+#endif // OPAL_IAX2
+
 /* The comment below is magic for those who use emacs to edit this file. */
 /* With the comment below, the tab key does auto indent to 4 spaces.     */
 
