@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21369 $
+ * $Revision: 21626 $
  * $Author: rjongbloed $
- * $Date: 2008-10-16 10:04:21 +0000 (Thu, 16 Oct 2008) $
+ * $Date: 2008-11-26 02:09:35 +0000 (Wed, 26 Nov 2008) $
  */
 
 #include <ptlib.h>
@@ -114,9 +114,16 @@ PBoolean OpalLineEndPoint::MakeConnection(OpalCall & call,
   
   // Locate a line
   OpalLine * line = GetLine(lineName, true, terminating);
-  if (line == NULL)
+  if (line == NULL && lineName != defaultLine) {
+    PTRACE(1,"LID EP\tMakeConnection cannot find the line \"" << lineName << '"');
+    line = GetLine(defaultLine, true, terminating);
+  }  
+  if (line == NULL){
+    PTRACE(1,"LID EP\tMakeConnection cannot find the default line " << defaultLine);
     return false;
 
+  }
+
   return AddConnection(CreateConnection(call, *line, userData, number));
 }
 
@@ -352,7 +359,7 @@ OpalLine * OpalLineEndPoint::GetLine(const PString & lineName, bool enableAudio,
     PString lineToken = line->GetToken();
     if (lineName != defaultLine && lineToken != lineName)
       PTRACE(4, "LID EP\tNo match to line name=\"" << lineToken << "\", default=" << defaultLine);
-    else if (line->IsTerminal() != terminating)
+    else if (line->IsTerminal() == terminating)
       PTRACE(4, "LID EP\tNo match to line name=\"" << lineToken << "\", terminating=" << line->IsTerminal());
     else if (!line->IsPresent())
       PTRACE(4, "LID EP\tNo match to line name=\"" << lineToken << "\", not present");
@@ -580,7 +587,8 @@ OpalMediaStream * OpalLineConnection::CreateMediaStream(const OpalMediaFormat &
                                                         unsigned sessionID,
                                                         PBoolean isSource)
 {
-  if (mediaFormat.GetMediaType() != OpalMediaType::Audio())
+  OpalMediaFormatList formats = line.GetDevice().GetMediaFormats();
+  if (formats.FindFormat(mediaFormat) == formats.end())
     return OpalConnection::CreateMediaStream(mediaFormat, sessionID, isSource);
 
   return new OpalLineMediaStream(*this, mediaFormat, sessionID, isSource, line);
@@ -872,6 +880,11 @@ PBoolean OpalLineConnection::SetUpConnection()
       return true;
     }
 
+    if (line.IsToneDetected() == OpalLineInterfaceDevice::BusyTone) {
+      Release(OpalConnection::EndedByRemoteBusy);
+      return false;
+    }
+
     PThread::Sleep(100);
   }
 
@@ -880,7 +893,7 @@ PBoolean OpalLineConnection::SetUpConnection()
          << ") on line " << line);
 
   if (m_dialParams.m_requireTones) {
-    Release(OpalConnection::EndedByNoAnswer);
+    Release(OpalConnection::EndedByRemoteBusy);
     return false;
   }
 
