@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21110 $
- * $Author: rjongbloed $
- * $Date: 2008-09-21 23:56:12 +0000 (Sun, 21 Sep 2008) $
+ * $Revision: 21851 $
+ * $Author: csoutheren $
+ * $Date: 2008-12-22 01:35:12 +0000 (Mon, 22 Dec 2008) $
  */
 
 #include <ptlib.h>
@@ -52,6 +52,10 @@
 #include <t38/t38proto.h>
 #include <ptclib/url.h>
 
+#if OPAL_HAS_IM
+#include <im/sipim.h>
+#include <im/rfc4103.h>
+#endif
 
 #define new PNEW
 
@@ -159,7 +163,7 @@ OpalConnection::OpalConnection(OpalCall & call,
                                OpalEndPoint  & ep,
                                const PString & token,
                                unsigned int options,
-                               OpalConnection::StringOptions * _stringOptions)
+                               OpalConnection::StringOptions * stringOptions)
   : PSafeObject(&call)  // Share the lock flag from the call
   , ownerCall(call)
   , endpoint(ep)
@@ -178,7 +182,13 @@ OpalConnection::OpalConnection(OpalCall & call,
   , q931Cause(0x100)
   , silenceDetector(NULL)
   , echoCanceler(NULL)
-  , stringOptions((_stringOptions == NULL) ? NULL : new OpalConnection::StringOptions(*_stringOptions))
+#ifdef _MSC_VER
+#pragma warning(disable:4355)
+#endif
+  , recordNotifier(PCREATE_NOTIFIER(OnRecordAudio))
+#ifdef _MSC_VER
+#pragma warning(default:4355)
+#endif
 #if OPAL_STATISTICS
   , m_VideoUpdateRequestsSent(0)
 #endif
@@ -189,6 +199,9 @@ OpalConnection::OpalConnection(OpalCall & call,
 
   ownerCall.connectionsActive.Append(this);
 
+  if (stringOptions != NULL)
+    m_connStringOptions = *stringOptions;
+
   detectInBandDTMF = !endpoint.GetManager().DetectInBandDTMFDisabled();
   minAudioJitterDelay = endpoint.GetManager().GetMinAudioJitterDelay();
   maxAudioJitterDelay = endpoint.GetManager().GetMaxAudioJitterDelay();
@@ -211,24 +224,6 @@ OpalConnection::OpalConnection(OpalCall & call,
       sendUserInputMode = ep.GetSendUserInputMode();
       break;
   }
-  
-  if (stringOptions != NULL) {
-    PString str((*stringOptions)("Call-Identifier"));
-    if (!str.IsEmpty())
-      callIdentifier = PGloballyUniqueID(str);
-
-    str = (*stringOptions)("enableinbanddtmf");
-    if (!str.IsEmpty())
-      detectInBandDTMF = str *= "true";
-    str = (*stringOptions)("dtmfmult");
-    if (!str.IsEmpty()) {
-      dtmfScaleMultiplier = str.AsInteger();
-      dtmfScaleDivisor    = 1;
-    }
-    str = (*stringOptions)("dtmfdiv");
-    if (!str.IsEmpty())
-      dtmfScaleDivisor = str.AsInteger();
-  }
 }
 
 OpalConnection::~OpalConnection()
@@ -240,7 +235,6 @@ OpalConnection::~OpalConnection()
 #if OPAL_T120DATA
   delete t120handler;
 #endif
-  delete stringOptions;
 
   ownerCall.connectionsActive.Remove(this);
   ownerCall.SafeDereference();
@@ -445,11 +439,8 @@ void OpalConnection::AnsweringCall(AnswerCallResponse response)
   PTRACE(3, "OpalCon\tAnswering call: " << response);
 
   PSafeLockReadWrite safeLock(*this);
-  if (!safeLock.IsLocked() || GetPhase() >= ReleasingPhase)
-    return;
-
-  // Can only answer call in these two phases
-  if (GetPhase() != SetUpPhase && GetPhase() != AlertingPhase)
+  // Can only answer call in UninitialisedPhase, SetUpPhase and AlertingPhase phases
+  if (!safeLock.IsLocked() || GetPhase() > AlertingPhase)
     return;
 
   switch (response) {
@@ -660,7 +651,7 @@ OpalMediaStream * OpalConnection::CreateMediaStream(
   )
 {
 #if OPAL_VIDEO
-  if (sessionID == OpalMediaFormat::DefaultVideoSessionID) {
+  if (mediaFormat.GetMediaType() == OpalMediaType::Video()) {
     if (isSource) {
       PVideoInputDevice * videoDevice;
       PBoolean autoDelete;
@@ -723,11 +714,11 @@ void OpalConnection::EnableRecording()
   if (!LockReadWrite())
     return;
 
-  OpalMediaStreamPtr stream = GetMediaStream(OpalMediaFormat::DefaultAudioSessionID, PTrue);
+  OpalMediaStreamPtr stream = GetMediaStream(OpalMediaType::Audio(), true);
   if (stream != NULL) {
     OpalMediaPatch * patch = stream->GetPatch();
     if (patch != NULL)
-      patch->AddFilter(PCREATE_NOTIFIER(OnRecordAudio), OPAL_PCM16);
+      patch->AddFilter(recordNotifier, OPAL_PCM16);
   }
   
   UnlockReadWrite();
@@ -738,11 +729,11 @@ void OpalConnection::DisableRecording()
   if (!LockReadWrite())
     return;
 
-  OpalMediaStreamPtr stream = GetMediaStream(OpalMediaFormat::DefaultAudioSessionID, PTrue);
+  OpalMediaStreamPtr stream = GetMediaStream(OpalMediaType::Audio(), true);
   if (stream != NULL) {
     OpalMediaPatch * patch = stream->GetPatch();
     if (patch != NULL)
-      patch->RemoveFilter(PCREATE_NOTIFIER(OnRecordAudio), OPAL_PCM16);
+      patch->RemoveFilter(recordNotifier, OPAL_PCM16);
   }
   
   UnlockReadWrite();
@@ -758,10 +749,10 @@ void OpalConnection::AttachRFC2833HandlerToPatch(PBoolean /*isSource*/, OpalMedi
 }
 
 
-OpalMediaStreamPtr OpalConnection::GetMediaStream(const PString & streamID) const
+OpalMediaStreamPtr OpalConnection::GetMediaStream(const PString & streamID, bool source) const
 {
   for (PSafePtr<OpalMediaStream> mediaStream(mediaStreams, PSafeReference); mediaStream != NULL; ++mediaStream) {
-    if (mediaStream->GetID() == streamID)
+    if (mediaStream->GetID() == streamID && mediaStream->IsSource() == source)
       return mediaStream;
   }
 
@@ -1086,36 +1077,56 @@ void OpalConnection::SetPhase(Phases phaseToSet)
 }
 
 
-PBoolean OpalConnection::OnOpenIncomingMediaChannels()
+void OpalConnection::SetStringOptions(const StringOptions & options, bool overwrite)
 {
-  return true;
-}
-
-
-void OpalConnection::SetStringOptions(StringOptions * options)
-{
-  if (LockReadWrite()) {
-    if (stringOptions != NULL)
-      delete stringOptions;
-    stringOptions = options;
-    UnlockReadWrite();
+  if (overwrite)
+    m_connStringOptions = options;
+  else {
+    for (PINDEX i = 0; i < options.GetSize(); ++i)
+      m_connStringOptions.SetAt(options.GetKeyAt(i), options.GetDataAt(i));
   }
 }
 
-
-void OpalConnection::ApplyStringOptions()
+void OpalConnection::OnApplyStringOptions()
 {
-  if (stringOptions != NULL && LockReadWrite()) {
-    if (stringOptions->Contains("Disable-Jitter"))
+  endpoint.GetManager().OnApplyStringOptions(*this, m_connStringOptions);
+}
+
+void OpalConnection::ApplyStringOptions(OpalConnection::StringOptions & stringOptions)
+{
+  if (LockReadWrite()) {
+
+    m_connStringOptions = stringOptions;
+  
+    PString str(stringOptions("Call-Identifier"));
+    if (!str.IsEmpty())
+      callIdentifier = PGloballyUniqueID(str);
+
+    str = stringOptions("enableinbanddtmf");
+    if (!str.IsEmpty())
+      detectInBandDTMF = str *= "true";
+    str = stringOptions("dtmfmult");
+    if (!str.IsEmpty()) {
+      dtmfScaleMultiplier = str.AsInteger();
+      dtmfScaleDivisor    = 1;
+    }
+    str = stringOptions("dtmfdiv");
+    if (!str.IsEmpty())
+      dtmfScaleDivisor = str.AsInteger();
+
+    m_autoStartInfo.Initialise(stringOptions);
+
+    if (stringOptions.Contains("Disable-Jitter"))
       maxAudioJitterDelay = minAudioJitterDelay = 0;
-    PString str = (*stringOptions)("Max-Jitter");
+    str = stringOptions("Max-Jitter");
     if (!str.IsEmpty())
       maxAudioJitterDelay = str.AsUnsigned();
-    str = (*stringOptions)("Min-Jitter");
+    str = stringOptions("Min-Jitter");
     if (!str.IsEmpty())
       minAudioJitterDelay = str.AsUnsigned();
-    if (stringOptions->Contains("Record-Audio"))
-      recordAudioFilename = (*stringOptions)("Record-Audio");
+    if (stringOptions.Contains("Record-Audio"))
+      recordAudioFilename = m_connStringOptions("Record-Audio");
+
     UnlockReadWrite();
   }
 }
@@ -1147,4 +1158,161 @@ void OpalConnection::OnMediaCommand(OpalMediaCommand & /*command*/, INT /*extra*
 }
 
 
+OpalMediaType::AutoStartMode OpalConnection::GetAutoStart(const OpalMediaType & mediaType) const
+{
+  return m_autoStartInfo.GetAutoStart(mediaType);
+}
+
+
+OpalConnection::AutoStartMap::AutoStartMap()
+{
+  m_initialised = false;
+}
+
+
+void OpalConnection::AutoStartMap::Initialise(const OpalConnection::StringOptions & stringOptions)
+{
+  PWaitAndSignal m(m_mutex);
+
+  // make function idempotent
+  if (m_initialised)
+    return;
+
+  m_initialised = true;
+
+  // get autostart option as lines
+  PStringArray lines = stringOptions("autostart").Lines();
+  for (PINDEX i = 0; i < lines.GetSize(); ++i) {
+    PString line = lines[i];
+    PINDEX colon = line.Find(':');
+    OpalMediaType mediaType = line.Left(colon);
+
+    // see if media type is known, and if it is, enable it
+    OpalMediaTypeDefinition * def = mediaType.GetDefinition();
+    if (def != NULL) {
+      if (colon == P_MAX_INDEX) 
+        SetAutoStart(mediaType, OpalMediaType::ReceiveTransmit);
+      else {
+        PStringArray tokens = line.Mid(colon+1).Tokenise(";", FALSE);
+        PINDEX j;
+        for (j = 0; j < tokens.GetSize(); ++j) {
+          if ((tokens[i] *= "no") || (tokens[i] *= "false") || (tokens[i] *= "0"))
+            SetAutoStart(mediaType, OpalMediaType::DontOffer);
+          else if ((tokens[i] *= "yes") || (tokens[i] *= "true") || (tokens[i] *= "1") || (tokens[i] *= "sendrecv"))
+            SetAutoStart(mediaType, OpalMediaType::ReceiveTransmit);
+          else if (tokens[i] *= "recvonly")
+            SetAutoStart(mediaType, OpalMediaType::Receive);
+          else if (tokens[i] *= "sendonly")
+            SetAutoStart(mediaType, OpalMediaType::Transmit);
+          else if (tokens[i] *= "offer")
+            SetAutoStart(mediaType, OpalMediaType::OfferInactive);
+          else if (tokens[i] *= "exclusive") {
+            OpalMediaTypeFactory::KeyList_T types = OpalMediaType::GetList();
+            for (OpalMediaTypeFactory::KeyList_T::iterator r = types.begin(); r != types.end(); ++r)
+              SetAutoStart(*r, *r == mediaType ? OpalMediaType::ReceiveTransmit : OpalMediaType::DontOffer);
+          }
+        }
+      }
+    }
+  }
+}
+
+
+void OpalConnection::AutoStartMap::SetAutoStart(const OpalMediaType & mediaType, OpalMediaType::AutoStartMode autoStart)
+{
+  PWaitAndSignal m(m_mutex);
+  m_initialised = true;
+
+  // deconflict session ID
+  unsigned sessionID = mediaType.GetDefinition()->GetDefaultSessionId();
+  if (size() == 0) {
+    if (sessionID == 0) 
+      sessionID = 1;
+  }
+  else {
+    iterator r = begin();
+    while (r != end()) {
+      if (r->second.preferredSessionId != sessionID) 
+        ++r;
+      else {
+        ++sessionID;
+        r = begin();
+      }
+    }
+  }
+
+  AutoStartInfo info;
+  info.autoStart = autoStart;
+  info.preferredSessionId = sessionID;
+
+  insert(value_type(mediaType, info));
+}
+
+
+OpalMediaType::AutoStartMode OpalConnection::AutoStartMap::GetAutoStart(const OpalMediaType & mediaType) const
+{
+  PWaitAndSignal m(m_mutex);
+  const_iterator r = find(mediaType);
+  return r == end() ? mediaType.GetAutoStart() : r->second.autoStart;
+}
+
+
+#if OPAL_HAS_IM
+
+template <class DataType>
+bool SendIMHelper(OpalConnection & conn, const OpalMediaFormat & format, const DataType & data)
+{
+  if (!conn.LockReadWrite())
+    return false;
+
+  bool stat = true;
+
+  OpalMediaStreamPtr strm = conn.GetMediaStream(format.GetMediaType(), true);
+  if (strm == NULL) 
+    stat = false;
+  else {
+    OpalIMMediaStream * imStream = dynamic_cast<OpalIMMediaStream *>(&*strm);
+    if (imStream != NULL) 
+      imStream->PushIM(data);
+  }
+  
+  conn.UnlockReadWrite();
+
+  return stat;
+}
+
+bool OpalConnection::SendIM(const OpalMediaFormat & format, const T140String & text)
+{
+  return SendIMHelper<T140String>(*this, format, text);
+}
+
+bool OpalConnection::SendIM(const OpalMediaFormat & format, const RTP_DataFrame & frame)
+{
+  return SendIMHelper<RTP_DataFrame>(*this, format, frame);
+}
+
+void OpalConnection::OnReceiveIM(const IMInfo & im)
+{
+  if (!LockReadWrite())
+    return;
+
+  for (PList<PNotifier>::iterator f = m_imListeners.begin(); f != m_imListeners.end(); ++f) {
+    (*f)((IMInfo &)im, (INT)this);
+  }
+
+  UnlockReadWrite();
+}
+
+void OpalConnection::AddIMListener(const PNotifier & listener)
+{
+  if (!LockReadWrite())
+    return;
+
+  m_imListeners.Append(new PNotifier(listener));
+
+  UnlockReadWrite();
+}
+
+#endif
+
 /////////////////////////////////////////////////////////////////////////////
