@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 21810 $
+ * $Author: csoutheren $
+ * $Date: 2008-12-16 08:12:08 +0000 (Tue, 16 Dec 2008) $
  */
 
 #include <ptlib.h>
@@ -49,15 +49,15 @@
 
 OpalIVREndPoint::OpalIVREndPoint(OpalManager & mgr, const char * prefix)
   : OpalEndPoint(mgr, prefix, CanTerminateCall),
-    defaultVXML("<?xml version=\"1.0\"?>"
-                "<vxml version=\"1.0\">"
-                  "<form id=\"root\">"
-                    "<audio src=\"welcome.wav\">"
-                      "This is the OPAL, V X M L test program, please speak after the tone."
-                    "</audio>"
-                    "<record name=\"msg\" beep=\"true\" dtmfterm=\"true\" dest=\"recording.wav\" maxtime=\"10s\"/>"
-                  "</form>"
-                "</vxml>")
+    defaultVXML("<?xml version=\"1.0\"?>\n"
+                "<vxml version=\"1.0\">\n"
+                "  <form id=\"root\">\n"
+                "    <audio src=\"welcome.wav\">\n"
+                "      This is the OPAL, V X M L test program, please speak after the tone.\n"
+                "    </audio>\n"
+                "    <record name=\"msg\" beep=\"true\" dtmfterm=\"true\" dest=\"recording.wav\" maxtime=\"10s\"/>\n"
+                "  </form>\n"
+                "</vxml>\n")
 {
   nextTokenNumber = 1;
 
@@ -159,7 +159,7 @@ OpalIVRConnection::OpalIVRConnection(OpalCall & call,
 #pragma warning(push)
 #pragma warning(disable:4355)
 #endif
-    vxmlSession(this, PFactory<PTextToSpeech>::CreateInstance(ep.GetDefaultTextToSpeech()))
+    vxmlSession(this, PFactory<PTextToSpeech>::CreateInstance(ep.GetDefaultTextToSpeech()), true)
 #ifdef _MSC_VER
 #pragma warning(pop)
 #endif
@@ -184,7 +184,7 @@ PBoolean OpalIVRConnection::SetUpConnection()
 {
   originating = false;
 
-  ApplyStringOptions();
+  OnApplyStringOptions();
 
   PTRACE(3, "IVR\tSetUpConnection(" << remotePartyName << ')');
 
@@ -209,14 +209,12 @@ PBoolean OpalIVRConnection::StartVXML()
 {
   PStringToString & vars = vxmlSession.GetSessionVars();
 
-  if (stringOptions != NULL) {
-    PString originator = (*stringOptions)("Originator-Address");
-    if (originator.IsEmpty())
-      originator = (*stringOptions)("Remote-Address");
-    if (!originator.IsEmpty()) {
-      PIPSocketAddressAndPort ap(originator);
-      vars.SetAt("Source-IP-Address", ap.address.AsString());
-    }
+  PString originator = m_connStringOptions("Originator-Address");
+  if (originator.IsEmpty())
+    originator = m_connStringOptions("Remote-Address");
+  if (!originator.IsEmpty()) {
+    PIPSocketAddressAndPort ap(originator);
+    vars.SetAt("Source-IP-Address", ap.GetAddress().AsString());
   }
 
   vars.SetAt("Time", PTime().AsString());
