@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21004 $
- * $Author: rjongbloed $
- * $Date: 2008-09-16 07:08:56 +0000 (Tue, 16 Sep 2008) $
+ * $Revision: 21862 $
+ * $Author: csoutheren $
+ * $Date: 2008-12-23 03:24:53 +0000 (Tue, 23 Dec 2008) $
  */
 
 #include <ptlib.h>
@@ -47,8 +47,9 @@
 #include <codec/vidcodec.h>
 #endif
 
-#include <h224/h224handler.h>
-#include <h224/h281handler.h>
+#if OPAL_HAS_H224
+#include <h224/h224.h>
+#endif
 
 #include <ptclib/random.h>
 
@@ -66,7 +67,11 @@ static const char * const DefaultMediaFormatOrder[] = {
   OPAL_GSM0610,
   OPAL_G728,
   OPAL_G711_ULAW_64K,
-  OPAL_G711_ALAW_64K
+  OPAL_G711_ALAW_64K,
+  OPAL_H264,
+  OPAL_MPEG4,
+  OPAL_H263,
+  OPAL_H261
 };
 
 #define new PNEW
@@ -158,8 +163,8 @@ OpalManager::OpalManager()
   , interfaceMonitor(NULL)
   , activeCalls(*this)
   , clearingAllCalls(PFalse)
-#if OPAL_RTP_AGGREGATE
-  , useRTPAggregation(false)
+#ifdef OPAL_ZRTP
+  , zrtpEnabled(false)
 #endif
 {
   rtpIpPorts.current = rtpIpPorts.base = 5000;
@@ -169,7 +174,7 @@ OpalManager::OpalManager()
   tcpPorts.current = tcpPorts.base = tcpPorts.max = 0;
   udpPorts.current = udpPorts.base = udpPorts.max = 0;
 
-#if OPAL_PTLIB_VIDEO
+#if OPAL_VIDEO
   PStringArray devices = PVideoInputDevice::GetDriversDeviceNames("*"); // Get all devices on all drivers
   PINDEX i;
   for (i = 0; i < devices.GetSize(); ++i) {
@@ -178,19 +183,17 @@ OpalManager::OpalManager()
     videoInputDevice.deviceName = devices[i];
     break;
   }
-  autoStartTransmitVideo = !videoInputDevice.deviceName.IsEmpty();
+  SetAutoStartTransmitVideo(!videoInputDevice.deviceName.IsEmpty());
 
   devices = PVideoOutputDevice::GetDriversDeviceNames("*"); // Get all devices on all drivers
   for (i = 0; i < devices.GetSize(); ++i) {
     if ((devices[i] *= "*.yuv") || (devices[i] *= "null"))
       continue;
     videoOutputDevice.deviceName = devices[i];
+    videoPreviewDevice = videoOutputDevice;
     break;
   }
-  autoStartReceiveVideo = !videoOutputDevice.deviceName.IsEmpty();
-
-  if (autoStartReceiveVideo)
-    videoPreviewDevice = videoOutputDevice;
+  SetAutoStartReceiveVideo(!videoOutputDevice.deviceName.IsEmpty());
 #endif
 
   garbageCollector = PThread::Create(PCREATE_NOTIFIER(GarbageMain), "Opal Garbage");
@@ -488,7 +491,11 @@ PString OpalManager::GetNextCallToken()
   return psprintf("%c%08x%u", PRandom::Number('a', 'z'), PRandom::Number(), ++lastCallTokenID);
 }
 
-PBoolean OpalManager::MakeConnection(OpalCall & call, const PString & remoteParty, void * userData, unsigned int options, OpalConnection::StringOptions * stringOptions)
+PBoolean OpalManager::MakeConnection(OpalCall & call,
+                                     const PString & remoteParty,
+                                     void * userData,
+                                     unsigned int options,
+                                     OpalConnection::StringOptions * stringOptions)
 {
   PTRACE(3, "OpalMan\tSet up connection to \"" << remoteParty << '"');
 
@@ -530,7 +537,9 @@ PBoolean OpalManager::OnIncomingConnection(OpalConnection & /*connection*/, unsi
 
 PBoolean OpalManager::OnIncomingConnection(OpalConnection & connection, unsigned options, OpalConnection::StringOptions * stringOptions)
 {
-  PTRACE(3, "OpalMan\tOn incoming connection " << connection);
+  PTRACE(3, "OpalMan\tOnIncoming connection " << connection);
+
+  connection.OnApplyStringOptions();
 
   if (!OnIncomingConnection(connection))
     return PFalse;
@@ -708,7 +717,9 @@ PBoolean OpalManager::CreateVideoInputDevice(const OpalConnection & /*connection
   PVideoDevice::OpenArgs args = videoInputDevice;
   args.width = mediaFormat.GetOptionInteger(OpalVideoFormat::FrameWidthOption(), PVideoFrameInfo::QCIFWidth);
   args.height = mediaFormat.GetOptionInteger(OpalVideoFormat::FrameHeightOption(), PVideoFrameInfo::QCIFHeight);
-  args.rate = mediaFormat.GetClockRate()/mediaFormat.GetFrameTime();
+  unsigned maxRate = mediaFormat.GetClockRate()/mediaFormat.GetFrameTime();
+  if (args.rate > maxRate)
+    args.rate = maxRate;
 
   autoDelete = PTrue;
   device = PVideoInputDevice::CreateOpenedDevice(args, false);
@@ -724,7 +735,10 @@ PBoolean OpalManager::CreateVideoOutputDevice(const OpalConnection & connection,
                                           PBoolean & autoDelete)
 {
   // Donot use our one and only SDL window, if we need it for the video output.
-  if (preview && videoPreviewDevice.driverName == "SDL" && videoOutputDevice.driverName == "SDL")
+  if (preview && (
+      (videoPreviewDevice.driverName == "SDL" && videoOutputDevice.driverName == "SDL") ||
+      (videoPreviewDevice.deviceName == "SDL" && videoOutputDevice.deviceName == "SDL")
+      ))
     return PFalse;
 
   // Make copy so we can adjust the size
@@ -1202,18 +1216,21 @@ PBoolean OpalManager::IsRTPNATEnabled(OpalConnection & /*conn*/,
 PBoolean OpalManager::TranslateIPAddress(PIPSocket::Address & localAddress,
                                      const PIPSocket::Address & remoteAddress)
 {
-  if (!translationAddress.IsValid())
-    return PFalse; // Have nothing to translate it to
-
   if (!IsLocalAddress(localAddress))
-    return PFalse; // Is already translated
+    return false; // Is already translated
 
   if (IsLocalAddress(remoteAddress))
-    return PFalse; // Does not need to be translated
+    return false; // Does not need to be translated
 
-  // Tranlsate it!
-  localAddress = translationAddress;
-  return PTrue;
+  if (translationAddress.IsValid()) {
+    localAddress = translationAddress; // Translate it!
+    return true;
+  }
+
+  if (stun != NULL && stun->GetNatType() != PSTUNClient::BlockedNat)
+    return stun->GetExternalAddress(localAddress); // Translate it!
+
+  return false; // Have nothing to translate it to
 }
 
 
@@ -1271,10 +1288,11 @@ PSTUNClient::NatTypes OpalManager::SetSTUNServer(const PString & server)
   }
 
   PSTUNClient::NatTypes type = stun->GetNatType();
+  PIPSocket::Address stunExternalAddress;
   if (type != PSTUNClient::BlockedNat)
-    stun->GetExternalAddress(translationAddress);
+    stun->GetExternalAddress(stunExternalAddress);
 
-  PTRACE(3, "OPAL\tSTUN server \"" << server << "\" replies " << type << ", external IP " << translationAddress);
+  PTRACE(3, "OPAL\tSTUN server \"" << server << "\" replies " << type << ", external IP " << stunExternalAddress);
 
   return type;
 }
@@ -1497,15 +1515,6 @@ void OpalManager::OnNewConnection(OpalConnection & /*conn*/)
 {
 }
 
-PBoolean OpalManager::UseRTPAggregation() const
-{ 
-#if OPAL_RTP_AGGREGATE
-  return useRTPAggregation; 
-#else
-  return PFalse;
-#endif
-}
-
 PBoolean OpalManager::StartRecording(const PString & callToken, const PFilePath & fn)
 {
   PSafePtr<OpalCall> call = activeCalls.FindWithLock(callToken, PSafeReadWrite);
@@ -1531,6 +1540,31 @@ void OpalManager::StopRecording(const PString & callToken)
 }
 
 
+void OpalManager::AddIMMediaFormats(OpalMediaFormatList & mediaFormats, const OpalConnection * /*connection*/) const
+{
+#if OPAL_HAS_MSRP
+  mediaFormats += OpalMSRP;
+#endif
+#if OPAL_HAS_SIPIM
+  mediaFormats += OpalSIPIM;
+#endif
+#if OPAL_HAS_RFC4103
+  mediaFormats += OpalT140;
+#endif
+#if OPAL_HAS_H224
+  mediaFormats += OpalH224AnnexQ;
+  mediaFormats += OpalH224Tunnelled;
+#endif
+}
+
+void OpalManager::OnApplyStringOptions(
+  OpalConnection & conn,
+  OpalConnection::StringOptions & stringOptions
+)
+{
+  conn.ApplyStringOptions(stringOptions);
+}
+
 /////////////////////////////////////////////////////////////////////////////
 
 OpalManager::InterfaceMonitor::InterfaceMonitor(OpalManager & manager)
@@ -1552,14 +1586,20 @@ void OpalManager::InterfaceMonitor::OnRemoveInterface(const PIPSocket::Interface
     stun->InvalidateCache();
 }
 
+#ifdef OPAL_ZRTP
+bool OpalManager::GetZRTPEnabled() const
+{
+  return zrtpEnabled;
+}
+#endif
 
 /////////////////////////////////////////////////////////////////////////////
 
 OpalRecordManager::Mixer_T::Mixer_T()
-  : OpalAudioMixer(PTrue)
-{ 
-  mono = PFalse; 
-  started = PFalse; 
+  :	OpalAudioMixer(true)
+  ,	mono(false)
+  ,	started(false)
+{
 }
 
 PBoolean OpalRecordManager::Mixer_T::Open(const PFilePath & fn)
@@ -1579,8 +1619,11 @@ PBoolean OpalRecordManager::Mixer_T::Open(const PFilePath & fn)
 PBoolean OpalRecordManager::Mixer_T::Close()
 {
   PWaitAndSignal m(mutex);
+  
+  started = false;
   file.Close();
-  return PTrue;
+
+  return true;
 }
 
 PBoolean OpalRecordManager::Mixer_T::OnWriteAudio(const MixerFrame & mixerFrame)
