@@ -23,9 +23,9 @@
  *
  * Contributor(s): Vyacheslav Frolov.
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 21865 $
+ * $Author: rjongbloed $
+ * $Date: 2008-12-23 05:18:47 +0000 (Tue, 23 Dec 2008) $
  */
 
 #include <ptlib.h>
@@ -64,7 +64,6 @@ class T38PseudoRTP_Handler : public RTP_Encoding
     {  
       RTP_Encoding::OnStart(_rtpUDP);
       rtpUDP->SetJitterBufferSize(0, 0);
-      corrigendumASN        = PTrue;
       consecutiveBadPackets = 0;
       oneGoodPacket         = false;
 
@@ -207,7 +206,6 @@ class T38PseudoRTP_Handler : public RTP_Encoding
 
 
   protected:
-    PBoolean corrigendumASN;
     int consecutiveBadPackets;
     bool oneGoodPacket;
     PBYTEArray lastIFP;
@@ -232,15 +230,20 @@ OpalFaxCallInfo::OpalFaxCallInfo()
 
 /////////////////////////////////////////////////////////////////////////////
 
-OpalFaxMediaStream::OpalFaxMediaStream(OpalConnection & conn, 
+OpalFaxMediaStream::OpalFaxMediaStream(OpalFaxConnection & conn, 
                                 const OpalMediaFormat & mediaFormat, 
                                                unsigned sessionID, 
-                                               PBoolean isSource, 
-                                       const PString & _token, 
-                                       const PString & _filename, 
-                                              PBoolean _receive,
-                                       const PString & _stationId)
-  : OpalMediaStream(conn, mediaFormat, sessionID, isSource), sessionToken(_token), filename(_filename), receive(_receive), stationId(_stationId)
+                                                   bool isSource, 
+                                       const PString & token, 
+                                       const PString & filename, 
+                                                  bool receive,
+                                       const PString & stationId)
+  : OpalMediaStream(conn, mediaFormat, sessionID, isSource)
+  , m_connection(conn)
+  , sessionToken(token)
+  , m_filename(filename)
+  , m_receive(receive)
+  , m_stationId(stationId)
 {
   faxCallInfo = NULL;
   SetDataSize(RTP_DataFrame::MaxMtuPayloadSize);
@@ -250,7 +253,7 @@ PBoolean OpalFaxMediaStream::Open()
 {
   if (sessionToken.IsEmpty()) {
     PTRACE(1, "T38\tCannot open unknown media stream");
-    return PFalse;
+    return false;
   }
 
   PWaitAndSignal m2(faxMapMutex);
@@ -265,7 +268,7 @@ PBoolean OpalFaxMediaStream::Open()
       faxCallInfo = new OpalFaxCallInfo();
       if (!faxCallInfo->socket.Listen()) {
         PTRACE(1, "Fax\tCannot open listening socket for SpanDSP");
-        return PFalse;
+        return false;
       }
       {
         PIPSocket::Address addr; WORD port;
@@ -295,7 +298,7 @@ PBoolean OpalFaxMediaStream::Open()
     // open connection to spandsp
     if (!faxCallInfo->spanDSP.Open(cmdLine, PPipeChannel::ReadOnly, false, true)) {
       PTRACE(1, "Fax\tCannot open SpanDSP: " << faxCallInfo->spanDSP.GetErrorText());
-      return PFalse;
+      return false;
     }
 
 #if PTRACING
@@ -306,18 +309,13 @@ PBoolean OpalFaxMediaStream::Open()
 
     if (!faxCallInfo->spanDSP.Execute()) {
       PTRACE(1, "Fax\tCannot execute SpanDSP: return code=" << faxCallInfo->spanDSP.GetReturnCode());
-      return PFalse;
+      return false;
     }
   }
 
   return OpalMediaStream::Open();
 }
 
-PBoolean OpalFaxMediaStream::Start()
-{
-  return OpalMediaStream::Start();
-}
-
 PBoolean OpalFaxMediaStream::ReadPacket(RTP_DataFrame & packet)
 {
 #if PTRACING
@@ -340,12 +338,12 @@ PBoolean OpalFaxMediaStream::ReadPacket(RTP_DataFrame & packet)
     if (faxCallInfo->spanDSPPort > 0) {
       if (!faxCallInfo->socket.Read(packet.GetPointer()+RTP_DataFrame::MinHeaderSize, packet.GetSize()-RTP_DataFrame::MinHeaderSize)) {
         faxCallInfo->socket.Close();
-        return PFalse;
+        return false;
       }
     } else{ 
       if (!faxCallInfo->socket.ReadFrom(packet.GetPointer()+RTP_DataFrame::MinHeaderSize, packet.GetSize()-RTP_DataFrame::MinHeaderSize, faxCallInfo->spanDSPAddr, faxCallInfo->spanDSPPort)) {
         faxCallInfo->socket.Close();
-        return PFalse;
+        return false;
       }
     }
 
@@ -353,6 +351,9 @@ PBoolean OpalFaxMediaStream::ReadPacket(RTP_DataFrame & packet)
     packet.SetPayloadType(RTP_DataFrame::MaxPayloadType);
     packet.SetPayloadSize(len);
 
+    if (len > 0)
+      m_connection.CheckFaxStopped();
+
 #if WRITE_PCM_FILE
     static int file = _open("t38_audio_in.pcm", _O_BINARY | _O_CREAT | _O_TRUNC | _O_WRONLY, _S_IREAD | _S_IWRITE);
     if (file >= 0) {
@@ -364,7 +365,7 @@ PBoolean OpalFaxMediaStream::ReadPacket(RTP_DataFrame & packet)
 #endif    
   }
 
-  return PTrue;
+  return true;
 }
 
 PBoolean OpalFaxMediaStream::WritePacket(RTP_DataFrame & packet)
@@ -379,7 +380,7 @@ PBoolean OpalFaxMediaStream::WritePacket(RTP_DataFrame & packet)
 
     if (!faxCallInfo->spanDSP.IsRunning()) {
       PTRACE(1, "Fax\tspandsp ended");
-      return PFalse;
+      return false;
     }
 
 #if WRITE_PCM_FILE
@@ -404,7 +405,7 @@ PBoolean OpalFaxMediaStream::WritePacket(RTP_DataFrame & packet)
         if (writeBufferLen == 0) {
           if (!faxCallInfo->socket.WriteTo(ptr, sizeof(writeBuffer), faxCallInfo->spanDSPAddr, faxCallInfo->spanDSPPort)) {
             PTRACE(1, "Fax\tSocket write error - " << faxCallInfo->socket.GetErrorText(PChannel::LastWriteError));
-            return PFalse;
+            return false;
           }
           len = sizeof(writeBuffer);
         }
@@ -413,7 +414,7 @@ PBoolean OpalFaxMediaStream::WritePacket(RTP_DataFrame & packet)
           memcpy(writeBuffer + writeBufferLen, ptr, len);
           if (!faxCallInfo->socket.WriteTo(writeBuffer, sizeof(writeBuffer), faxCallInfo->spanDSPAddr, faxCallInfo->spanDSPPort)) {
             PTRACE(1, "Fax\tSocket write error - " << faxCallInfo->socket.GetErrorText(PChannel::LastWriteError));
-            return PFalse;
+            return false;
           }
         }
         ptr += len;
@@ -430,19 +431,21 @@ PBoolean OpalFaxMediaStream::WritePacket(RTP_DataFrame & packet)
       if (writeBufferLen == sizeof(writeBuffer)) {
         if (!faxCallInfo->socket.WriteTo(writeBuffer, sizeof(writeBuffer), faxCallInfo->spanDSPAddr, faxCallInfo->spanDSPPort)) {
           PTRACE(1, "Fax\tSocket write error - " << faxCallInfo->socket.GetErrorText(PChannel::LastWriteError));
-          return PFalse;
+          return false;
         }
         writeBufferLen = 0;
       }
+
+      m_connection.CheckFaxStopped();
     }
   }
 
-  return PTrue;
+  return true;
 }
 
 PBoolean OpalFaxMediaStream::Close()
 {
-  PBoolean stat = OpalMediaStream::Close();
+  bool stat = OpalMediaStream::Close();
 
   PWaitAndSignal m2(faxMapMutex);
 
@@ -502,7 +505,7 @@ PBoolean OpalFaxMediaStream::Close()
 
 PBoolean OpalFaxMediaStream::IsSynchronous() const
 {
-  return PTrue;
+  return true;
 }
 
 PString OpalFaxMediaStream::GetSpanDSPCommandLine(OpalFaxCallInfo & info)
@@ -513,11 +516,11 @@ PString OpalFaxMediaStream::GetSpanDSPCommandLine(OpalFaxCallInfo & info)
   info.socket.GetLocalAddress(dummy, port);
 
   cmdline << ((OpalFaxEndPoint &)connection.GetEndPoint()).GetSpanDSP() << " -m ";
-  if (receive)
+  if (m_receive)
     cmdline << "fax_to_tiff";
   else
     cmdline << "tiff_to_fax";
-  cmdline << "-V 0 -n '" << filename << "' -f 127.0.0.1:" << port;
+  cmdline << " -V 0 -n '" << m_filename << "' -f 127.0.0.1:" << port;
 
   return cmdline;
 }
@@ -526,20 +529,19 @@ PString OpalFaxMediaStream::GetSpanDSPCommandLine(OpalFaxCallInfo & info)
 
 /**This class describes a media stream that transfers data to/from a T.38 session
   */
-OpalT38MediaStream::OpalT38MediaStream(
-      OpalConnection & conn,
-      const OpalMediaFormat & mediaFormat, ///<  Media format for stream
-      unsigned sessionID, 
-      PBoolean isSource,                       ///<  Is a source stream
-      const PString & token,               ///<  token used to match incoming/outgoing streams
-      const PString & _filename,
-      PBoolean _receive,
-      const PString & _stationId
-    )
-  : OpalFaxMediaStream(conn, mediaFormat, sessionID, isSource, token, _filename, _receive, _stationId)
+OpalT38MediaStream::OpalT38MediaStream(OpalFaxConnection     & conn,
+                                       const OpalMediaFormat & mediaFormat,
+                                       unsigned                sessionID, 
+                                       bool                    isSource,
+                                       const PString         & token,
+                                       const PString         & filename,
+                                       bool                    receive,
+                                       const PString         & stationId)
+  : OpalFaxMediaStream(conn, mediaFormat, sessionID, isSource, token, filename, receive, stationId)
 {
 }
 
+
 PString OpalT38MediaStream::GetSpanDSPCommandLine(OpalFaxCallInfo & info)
 {
   PStringStream cmdline;
@@ -548,18 +550,19 @@ PString OpalT38MediaStream::GetSpanDSPCommandLine(OpalFaxCallInfo & info)
   info.socket.GetLocalAddress(dummy, port);
 
   cmdline << ((OpalFaxEndPoint &)connection.GetEndPoint()).GetSpanDSP() << " -V 0 -m ";
-  if (receive)
+  if (m_receive)
     cmdline << "t38_to_tiff";
   else {
     cmdline << "tiff_to_t38";
-    if (!stationId.IsEmpty())
-      cmdline << " -s '" << stationId << "'";
+    if (!m_stationId.IsEmpty())
+      cmdline << " -s '" << m_stationId << "'";
   }
-  cmdline << " -v -n '" << filename << "' -t 127.0.0.1:" << port;
+  cmdline << " -v -n '" << m_filename << "' -t 127.0.0.1:" << port;
 
   return cmdline;
 }
 
+
 PBoolean OpalT38MediaStream::ReadPacket(RTP_DataFrame & packet)
 {
   // it is possible for ReadPacket to be called before the media stream has been opened, so deal with that case
@@ -593,15 +596,17 @@ PBoolean OpalT38MediaStream::ReadPacket(RTP_DataFrame & packet)
 
     PINDEX len = faxCallInfo->socket.GetLastReadCount();
     if (len < RTP_DataFrame::MinHeaderSize)
-      return PFalse;
+      return false;
 
     packet.SetSize(len);
     packet.SetPayloadSize(len - RTP_DataFrame::MinHeaderSize);
+    m_connection.CheckFaxStopped();
   }
 
-  return PTrue;
+  return true;
 }
 
+
 PBoolean OpalT38MediaStream::WritePacket(RTP_DataFrame & packet)
 {
   PWaitAndSignal m(infoMutex);
@@ -617,24 +622,27 @@ PBoolean OpalT38MediaStream::WritePacket(RTP_DataFrame & packet)
         RTP_DataFrame & frame = queuedFrames[i];
         if (!faxCallInfo->socket.WriteTo(frame.GetPointer(), frame.GetHeaderSize() + frame.GetPayloadSize(), faxCallInfo->spanDSPAddr, faxCallInfo->spanDSPPort)) {
           PTRACE(2, "T38_UDP\tSocket write error - " << faxCallInfo->socket.GetErrorText(PChannel::LastWriteError));
-          return PFalse;
+          return false;
         }
       }
       queuedFrames.RemoveAll();
     }
     if (!faxCallInfo->socket.WriteTo(packet.GetPointer(), packet.GetHeaderSize() + packet.GetPayloadSize(), faxCallInfo->spanDSPAddr, faxCallInfo->spanDSPPort)) {
       PTRACE(2, "T38_UDP\tSocket write error - " << faxCallInfo->socket.GetErrorText(PChannel::LastWriteError));
-      return PFalse;
+      return false;
     }
+    m_connection.CheckFaxStopped();
   }
 
-  return PTrue;
+  return true;
 }
 
+
 /////////////////////////////////////////////////////////////////////////////
 
-OpalFaxEndPoint::OpalFaxEndPoint(OpalManager & mgr, const char * prefix)
-  : OpalEndPoint(mgr, prefix, CanTerminateCall)
+OpalFaxEndPoint::OpalFaxEndPoint(OpalManager & mgr, const char * g711Prefix, const char * t38Prefix)
+  : OpalEndPoint(mgr, g711Prefix, CanTerminateCall)
+  , m_t38Prefix(t38Prefix)
 #ifdef _WIN32
   , m_spanDSP("./spandsp_util.exe")
 #else
@@ -642,32 +650,43 @@ OpalFaxEndPoint::OpalFaxEndPoint(OpalManager & mgr, const char * prefix)
 #endif
   , m_defaultDirectory(".")
 {
+  if (t38Prefix != NULL)
+    mgr.AttachEndPoint(this, m_t38Prefix);
+
   PTRACE(3, "Fax\tCreated Fax endpoint");
 }
 
+
 OpalFaxEndPoint::~OpalFaxEndPoint()
 {
   PTRACE(3, "Fax\tDeleted Fax endpoint.");
 }
 
-OpalFaxConnection * OpalFaxEndPoint::CreateConnection(OpalCall & call, const PString & filename, PBoolean receive, void * /*userData*/, OpalConnection::StringOptions * stringOptions)
+
+OpalFaxConnection * OpalFaxEndPoint::CreateConnection(OpalCall & call,
+                                                      void * /*userData*/,
+                                                      OpalConnection::StringOptions * stringOptions,
+                                                      const PString & filename,
+                                                      bool receive,
+                                                      bool t38)
 {
-  return new OpalFaxConnection(call, *this, filename, receive, MakeToken(), stringOptions);
+  PString token = t38 ? m_t38Prefix : GetPrefixName();
+  token.sprintf("%i", ++faxCallIndex);
+
+  if (t38)
+    return new OpalT38Connection(call, *this, filename, receive, token, stringOptions);
+  else
+    return new OpalFaxConnection(call, *this, filename, receive, token, stringOptions);
 }
 
+
 OpalMediaFormatList OpalFaxEndPoint::GetMediaFormats() const
 {
   OpalMediaFormatList formats;
-
-  //formats += OpalPCM16;        
-
+  formats += OpalPCM16;
   return formats;
 }
 
-PString OpalFaxEndPoint::MakeToken()
-{
-  return psprintf("FaxConnection_%i", ++faxCallIndex);
-}
 
 void OpalFaxEndPoint::AcceptIncomingConnection(const PString & token)
 {
@@ -676,18 +695,15 @@ void OpalFaxEndPoint::AcceptIncomingConnection(const PString & token)
     connection->AcceptIncoming();
 }
 
+
 PBoolean OpalFaxEndPoint::MakeConnection(OpalCall & call,
                                 const PString & remoteParty,
                                          void * userData,
                                  unsigned int /*options*/,
                 OpalConnection::StringOptions * stringOptions)
 {
-  // First strip of the prefix if present
-  PINDEX prefixLength = 0;
-  if (remoteParty.Find(GetPrefixName()+":") == 0)
-    prefixLength = GetPrefixName().GetLength()+1;
-
-  PStringArray tokens = remoteParty.Mid(prefixLength).Tokenise(";", true);
+  PINDEX prefixLength = remoteParty.Find(':');
+  PStringArray tokens = remoteParty.Mid(prefixLength+1).Tokenise(";", true);
   if (tokens.IsEmpty()) {
     PTRACE(2, "Fax\tNo filename specified!");
     return false;
@@ -709,7 +725,7 @@ PBoolean OpalFaxEndPoint::MakeConnection(OpalCall & call,
 
   if (!receive && !PFile::Exists(filename)) {
     PTRACE(2, "Fax\tCannot find filename '" << filename << "'");
-    return PFalse;
+    return false;
   }
 
   if (stringOptions == NULL)
@@ -717,81 +733,78 @@ PBoolean OpalFaxEndPoint::MakeConnection(OpalCall & call,
   if ((*stringOptions)("stationid").IsEmpty())
     stringOptions->SetAt("stationid", stationId);
 
-  PSafePtr<OpalFaxConnection> connection = PSafePtrCast<OpalConnection, OpalFaxConnection>(GetConnectionWithLock(MakeToken()));
-  if (connection != NULL)
-    return PFalse;
-
-  connection = CreateConnection(call, filename, receive, userData, stringOptions);
-  if (connection == NULL)
-    return PFalse;
-
-  connectionsActive.SetAt(connection->GetToken(), connection);
-
-  return PTrue;
+  return AddConnection(CreateConnection(call, userData, stringOptions, filename, receive,
+                                        remoteParty.Left(prefixLength) *= m_t38Prefix));
 }
 
-void OpalFaxEndPoint::OnPatchMediaStream(const OpalFaxConnection & /*connection*/, PBoolean /*isSource*/, OpalMediaPatch & /*patch*/)
-{
-}
 
 /////////////////////////////////////////////////////////////////////////////
 
-OpalFaxConnection::OpalFaxConnection(OpalCall & call, OpalFaxEndPoint & ep, const PString & _filename, PBoolean _receive, const PString & _token, OpalConnection::StringOptions * _stringOptions)
-  : OpalConnection(call, ep, _token, 0, _stringOptions), endpoint(ep), filename(_filename), receive(_receive)
+OpalFaxConnection::OpalFaxConnection(OpalCall        & call,
+                                     OpalFaxEndPoint & ep,
+                                     const PString   & filename,
+                                     bool              receive,
+                                     const PString   & token,
+                                     OpalConnection::StringOptions * stringOptions)
+  : OpalConnection(call, ep, token, 0, stringOptions)
+  , m_endpoint(ep)
+  , m_filename(filename)
+  , m_receive(receive)
 {
   PTRACE(3, "FAX\tCreated FAX connection with token '" << callToken << "'");
-  SetPhase(SetUpPhase);
 
-  if (stringOptions != NULL) {
-    forceFaxAudio = stringOptions->Contains("Force-Fax-Audio");
-    stationId     = (*stringOptions)("stationid");
-  }
-
-  detectInBandDTMF = true;
+  m_faxStopped.SetNotifier(PCREATE_NOTIFIER(OnFaxStoppedTimeout));
 }
 
+
 OpalFaxConnection::~OpalFaxConnection()
 {
   PTRACE(3, "FAX\tDeleted FAX connection.");
 }
 
-OpalMediaStream * OpalFaxConnection::CreateMediaStream(const OpalMediaFormat & mediaFormat, unsigned sessionID, PBoolean isSource)
+
+void OpalFaxConnection::ApplyStringOptions(OpalConnection::StringOptions & stringOptions)
 {
-  // if creating an audio session, use a NULL stream
-  if (forceFaxAudio && (mediaFormat == OpalPCM16))
-    return new OpalFaxMediaStream(*this, mediaFormat, sessionID, isSource, GetToken(), filename, receive, stationId);
-  else
-    return new OpalNullMediaStream(*this, mediaFormat, sessionID, isSource, true);
+  m_stationId = stringOptions("stationid");
+  OpalConnection::ApplyStringOptions(stringOptions);
 }
 
+
+OpalMediaStream * OpalFaxConnection::CreateMediaStream(const OpalMediaFormat & mediaFormat, unsigned sessionID, bool isSource)
+{
+  return new OpalFaxMediaStream(*this, mediaFormat, sessionID, isSource, GetToken(), m_filename, m_receive, m_stationId);
+}
+
+
 PBoolean OpalFaxConnection::SetUpConnection()
 {
   // Check if we are A-Party in this call, so need to do things differently
   if (ownerCall.GetConnection(0) == this) {
     SetPhase(SetUpPhase);
 
-    if (!OnIncomingConnection(0, stringOptions)) {
+    OnApplyStringOptions();
+
+    if (!OnIncomingConnection(0, NULL)) {
       Release(EndedByCallerAbort);
-      return PFalse;
+      return false;
     }
 
     PTRACE(2, "FAX\tOutgoing call routed to " << ownerCall.GetPartyB() << " for " << *this);
     if (!ownerCall.OnSetUp(*this)) {
       Release(EndedByNoAccept);
-      return PFalse;
+      return false;
     }
 
-    return PTrue;
+    return true;
   }
 
   PTRACE(3, "FAX\tSetUpConnection(" << remotePartyName << ')');
   SetPhase(AlertingPhase);
-  //endpoint.OnShowIncoming(*this);
   OnAlerting();
 
   OnConnectedInternal();
 
-  return PTrue;
+  return true;
 }
 
 
@@ -800,28 +813,30 @@ PBoolean OpalFaxConnection::SetAlerting(const PString & calleeName, PBoolean)
   PTRACE(3, "Fax\tSetAlerting(" << calleeName << ')');
   SetPhase(AlertingPhase);
   remotePartyName = calleeName;
-  //return endpoint.OnShowOutgoing(*this);
-  return PTrue;
+  return true;
 }
 
 
 OpalMediaFormatList OpalFaxConnection::GetMediaFormats() const
 {
   OpalMediaFormatList formats;
-  formats += OpalPCM16;       
+  formats += OpalPCM16;
   return formats;
 }
 
+
 void OpalFaxConnection::AdjustMediaFormats(OpalMediaFormatList & mediaFormats) const
 {
-  endpoint.AdjustMediaFormats(*this, mediaFormats);
-  if (forceFaxAudio)
-    mediaFormats.Remove(PStringArray(OpalT38));
-}
+  // Remove everything but G.711 or T.38
+  OpalMediaFormatList::iterator i = mediaFormats.begin();
+  while (i != mediaFormats.end()) {
+    if (*i == OpalG711_ULAW_64K || *i == OpalG711_ALAW_64K || *i == OpalT38)
+      ++i;
+    else
+      mediaFormats -= *i++;
+  }
 
-void OpalFaxConnection::OnPatchMediaStream(PBoolean isSource, OpalMediaPatch & patch)
-{
-  endpoint.OnPatchMediaStream(*this, isSource, patch);
+  OpalConnection::AdjustMediaFormats(mediaFormats);
 }
 
 
@@ -833,203 +848,193 @@ void OpalFaxConnection::AcceptIncoming()
   }
 }
 
-/////////////////////////////////////////////////////////////////////////////
 
-OpalT38EndPoint::OpalT38EndPoint(OpalManager & manager, const char * prefix)
-  : OpalFaxEndPoint(manager, prefix)
+void OpalFaxConnection::CheckFaxStopped()
 {
+  m_faxStopped.SetInterval(0, 10);
 }
 
-OpalMediaFormatList OpalT38EndPoint::GetMediaFormats() const
+
+void OpalFaxConnection::OnFaxStoppedTimeout(PTimer &, INT)
 {
-  OpalMediaFormatList formats;
-
-  formats += OpalPCM16;        // need this so we get lead-in to T.38 sessions
-  formats += OpalT38;        
-
-  return formats;
+  PTRACE(3, "FAX\tTimeout on fax connection: " << GetToken());
+  Release();
 }
 
-OpalFaxConnection * OpalT38EndPoint::CreateConnection(OpalCall & call, const PString & filename, PBoolean receive, void * /*userData*/, OpalConnection::StringOptions * stringOptions)
-{
-  return new OpalT38Connection(call, *this, filename, receive, MakeToken(), stringOptions);
-}
-
-PString OpalT38EndPoint::MakeToken()
-{
-  return psprintf("T38Connection_%i", ++faxCallIndex);
-}
 
 /////////////////////////////////////////////////////////////////////////////
 
-OpalT38Connection::OpalT38Connection(OpalCall & call, OpalT38EndPoint & ep, const PString & _filename, PBoolean _receive, const PString & _token, OpalConnection::StringOptions * stringOptions)
-  : OpalFaxConnection(call, ep, _filename, _receive, _token, stringOptions), t38WaitMode(T38Mode_Auto)
+OpalT38Connection::OpalT38Connection(OpalCall        & call,
+                                     OpalFaxEndPoint & ep,
+                                     const PString   & filename,
+                                     bool              receive,
+                                     const PString   & token,
+                                     OpalConnection::StringOptions * stringOptions)
+  : OpalFaxConnection(call, ep, filename, receive, token, stringOptions)
+  , m_syncMode(Mode_Timeout)
+  , m_faxMode(false)
 {
   PTRACE(3, "FAX\tCreated T.38 connection with token '" << callToken << "'");
-  forceFaxAudio         = false;
-  currentMode = newMode = false;
-  modeChangeTriggered   = false;
-  faxStartup            = true;
-
-  faxTimer.SetNotifier(PCREATE_NOTIFIER(OnFaxChangeTimeout));
 }
 
-OpalT38Connection::~OpalT38Connection()
+
+void OpalT38Connection::ApplyStringOptions(OpalConnection::StringOptions & stringOptions)
 {
+  PCaselessString opt = stringOptions("Fax-Sync-Mode");
+  if (opt == "Wait")
+    m_syncMode = Mode_Wait;
+  else if (opt == "Timeout")
+    m_syncMode = Mode_Timeout;
+  else if (opt == "UserInput")
+    m_syncMode = Mode_UserInput;
+  else if (opt == "InBand")
+    m_syncMode = Mode_InBand;
+
+  OpalFaxConnection::ApplyStringOptions(stringOptions);
 }
 
+
+void OpalT38Connection::OnEstablished()
+{
+  OpalFaxConnection::OnEstablished();
+
+  if (m_faxMode)
+    return;
+
+  switch (m_syncMode) {
+    case Mode_Timeout :
+      m_faxTimer.SetNotifier(PCREATE_NOTIFIER(OnFaxChangeTimeout));
+      m_faxTimer = 2000;
+      PTRACE(1, "T38\tStarting timer for mode change");
+      break;
+
+    case Mode_UserInput :
+    case Mode_InBand :
+      m_faxTimer.SetNotifier(PCREATE_NOTIFIER(OnSendCNGCED));
+      m_faxTimer = 1000;
+      PTRACE(1, "T38\tStarting timer for CNG/CED tone");
+      break;
+
+    default :
+      break;
+  }
+}
+
+
 OpalMediaStream * OpalT38Connection::CreateMediaStream(const OpalMediaFormat & mediaFormat, unsigned sessionID, PBoolean isSource)
 {
   // if creating an audio session, use a NULL stream
-  if (mediaFormat.GetMediaType() == OpalMediaType::Audio()) {
-    if (isSource)
-      InFaxMode(false);
+  if (mediaFormat.GetMediaType() == OpalMediaType::Audio())
     return new OpalNullMediaStream(*this, mediaFormat, sessionID, isSource, true);
-  }
 
   // if creating a T.38 stream, see what type it is
-  else if (mediaFormat.GetMediaType() == OpalMediaType::Fax()) {
-    if (isSource)
-      InFaxMode(true);
-    return new OpalT38MediaStream(*this, mediaFormat, sessionID, isSource, GetToken(), filename, receive, stationId);
-  }
+  if (mediaFormat.GetMediaType() == OpalMediaType::Fax())
+    return new OpalT38MediaStream(*this, mediaFormat, sessionID, isSource, GetToken(), m_filename, m_receive, m_stationId);
 
   return NULL;
 }
 
 
+void OpalT38Connection::OnClosedMediaStream(const OpalMediaStream & stream)
+{
+  m_faxMode = stream.GetMediaFormat().GetMediaType() != OpalMediaType::Fax();
+
+  if (m_syncMode == Mode_UserInput)
+    OnUserInputTone(' ', 0);
+  m_faxTimer.Stop();
+
+  OpalFaxConnection::OnClosedMediaStream(stream);
+}
+
+
 OpalMediaFormatList OpalT38Connection::GetMediaFormats() const
 {
   OpalMediaFormatList formats;
 
-  formats += OpalPCM16;        
-  formats += OpalT38;        
+  formats += OpalPCM16;
+  formats += OpalT38;
 
   return formats;
 }
 
 
-void OpalT38Connection::OnPatchMediaStream(PBoolean isSource, OpalMediaPatch & patch)
-{
-  OpalConnection::OnPatchMediaStream(isSource, patch);
-}
-
 PBoolean OpalT38Connection::SendUserInputTone(char tone, unsigned /*duration*/)
 {
-  if (((t38WaitMode & T38Mode_NSECED) != 0) && (tolower(tone) == 'y'))
+  if (!m_faxMode && m_syncMode != Mode_Wait && toupper(tone) == (m_receive ? 'X' : 'Y'))
     RequestFaxMode(true);
-
   return true;
 }
 
-void OpalT38Connection::OnFaxChangeTimeout(PTimer &, INT)
+
+void OpalT38Connection::OnSendCNGCED(PTimer & timer, INT)
 {
-  RequestFaxMode(true);
+  if (LockReadOnly() && !m_faxMode && m_syncMode == Mode_UserInput) {
+    if (m_receive) {
+      // Cadence for CED is single tone, but we repeat just in case
+      OnUserInputTone('Y', 3600);
+      timer = 5000;
+    }
+    else {
+      // Cadence for CNG is 500ms on 3 seconds off
+      OnUserInputTone('X', 500);
+      timer = 3000;
+    }
+    UnlockReadOnly();
+  }
 }
 
-struct ModeChangeRequestStruct {
-  OpalManager * manager;
-  PString callToken;
-  PString connectionToken;
-  bool newMode;
-};
 
-static void ReinviteFunction(ModeChangeRequestStruct request)
+void OpalT38Connection::OnFaxChangeTimeout(PTimer &, INT)
 {
-  PSafePtr<OpalCall> call = request.manager->FindCallWithLock(request.callToken);
-  unsigned otherIndex = (call->GetConnection(0)->GetToken() == request.connectionToken) ? 1 : 0;
+  if (LockReadWrite()) {
+    RequestFaxMode(true);
+    UnlockReadWrite();
+  }
+}
 
-  OpalMediaType mediaType = request.newMode ? OpalMediaType::Fax() : OpalMediaType::Audio();
 
-  PSafePtr<OpalConnection> otherParty = call->GetConnection(otherIndex, PSafeReadWrite);
+void OpalT38Connection::OpenFaxStreams(PThread &, INT)
+{
+  if (!LockReadWrite())
+    return;
+
+  OpalMediaFormat format = m_faxMode ? OpalT38 : OpalG711uLaw;
+  OpalMediaType mediaType = format.GetMediaType();
+
+  PSafePtr<OpalConnection> otherParty = ownerCall.GetOtherPartyConnection(*this);
   if (otherParty == NULL) {
     PTRACE(1, "T38\tCannot get other party for " << mediaType << " trigger");
   }
-  else if (!call->OpenSourceMediaStreams(*otherParty, mediaType, 1, request.newMode ? OpalT38 : OpalG711uLaw)) {
+  else if (!ownerCall.OpenSourceMediaStreams(*otherParty, mediaType, 1, format)) {
     PTRACE(1, "T38\tMode change request to " << mediaType << " failed");
   }
+
+  UnlockReadWrite();
 }
 
+
 void OpalT38Connection::RequestFaxMode(bool toFax)
 {
 #if PTRACING
   const char * modeStr = toFax ? "fax" : "audio";
 #endif
 
-  PWaitAndSignal m(modeMutex);
-
-  if (!modeChangeTriggered) {
-    if (toFax == currentMode) {
-      PTRACE(1, "T38\tIgnoring request for mode " << modeStr);
-      return;
-    }
-  }
-  else
-  {
-    if (toFax == newMode) {
-      PTRACE(1, "T38\tIgnoring duplicate request for mode " << modeStr);
-    } else {
-      PTRACE(1, "T38\tCan't change in-progress mode request for mode " << modeStr);
-    }
+  if (toFax == m_faxMode) {
+    PTRACE(1, "T38\tAlready in mode " << modeStr);
     return;
   }
 
   // definitely changing mode
-  faxTimer.Stop();
-  modeChangeTriggered = true;
-  newMode = toFax;
   PTRACE(1, "T38\tRequesting mode change to " << modeStr);
 
-  OpalCall & call = GetCall();
-  ModeChangeRequestStruct request;
-  request.manager         = &call.GetManager();
-  request.callToken       = call.GetToken();
-  request.connectionToken = GetToken();
-  request.newMode         = newMode;
+  m_faxMode = toFax;
+  m_faxTimer.Stop();
+  if (m_syncMode == Mode_UserInput)
+    OnUserInputTone('\0', 0);
 
-  new PThread1Arg<ModeChangeRequestStruct>(request, &ReinviteFunction, true);
+  PThread::Create(PCREATE_NOTIFIER(OpenFaxStreams));
 }
 
-void OpalT38Connection::InFaxMode(bool toFax)
-{
-#if PTRACING
-  const char * modeStr = toFax ? "fax" : "audio";
-#endif
-
-  PWaitAndSignal m(modeMutex);
-
-  modeChangeTriggered = false;
-  faxTimer.Stop();
-
-  if (!modeChangeTriggered) {
-    if (toFax == currentMode) {
-      if (!faxStartup) {
-        PTRACE(1, "T38\tUntriggered mode to same mode " << modeStr);
-      }
-      else {
-        if (faxStartup && !currentMode && (t38WaitMode & T38Mode_Timeout) != 0) {
-          faxTimer = receive ? 2000 : 8000;
-          PTRACE(1, "T38\tStarting timer for mode change");
-        }
-      }
-    }
-    else {
-      PTRACE(1, "T38\tMode changed to different mode " << modeStr);
-    }
-  }
-
-  else {
-    if (toFax == newMode) {
-      PTRACE(1, "T38\tMode changed to correct mode " << modeStr);
-    }
-    else {
-      PTRACE(1, "T38\tMode changed to wrong mode " << modeStr);
-    }
-  }
-
-  faxStartup = false;
-  currentMode = toFax;
-}
 
 #endif // OPAL_FAX
 
